(herald "DHCR: unified model (UM) original" (bound 20) (limit 2000)
  (algebra diffie-hellman))

(comment "CPSA 4.4.3")
(comment "All input read from tst/dhcr_um.scm")
(comment "Strand count bounded at 20")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (l x rndx) (gb gy base) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a l))
      (recv (sig (body b gb (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv (cat gy (enc na nb a b (hash (exp gb l) (exp gy x)))))
      (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x l))
    (gen-st (pv a l))
    (fn-of ("principal-of" (ltxa a) (ltxb b))
      ("ltx-of" (a ltxa) (b ltxb))))
  (defrole resp
    (vars (l y rndx) (ga gx base) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b l))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b gx))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp ga l) (exp gx y))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y l))
    (gen-st (pv b l))
    (fn-of ("principal-of" (ltxa a) (ltxb b))
      ("ltx-of" (a ltxa) (b ltxb))))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l)
    (fn-of ("principal-of" (l self)) ("ltx-of" (self l))))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l))
    (fn-of ("principal-of" (l self)) ("ltx-of" (self l))))
  (defrule fact-resp-neq0
    (forall ((z strd))
      (implies
        (and (p "resp" z (idx 3)) (p "resp" "gx" z (gen)))
        (false))))
  (defrule fact-init-neq0
    (forall ((z strd))
      (implies
        (and (p "init" z (idx 4)) (p "init" "gy" z (gen)))
        (false))))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (a name) (l rndx))
      (implies
        (and (p "init" z (idx 1)) (p "init" "l" z l) (p "init" "a" z a))
        (gen-st (pv a l)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (b name) (l rndx))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "l" z l) (p "resp" "b" z b))
        (gen-st (pv b l)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (self name) (l rndx))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (gy base) (priv-stor locn)
    (l l-peer x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-peer))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x l))
  (facts (neq a b) (undisclosed l) (undisclosed l-peer))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-peer) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b
            (hash (exp (gen) (mul l l-peer)) (exp gy x)))))))
  (label 0)
  (unrealized (0 1) (0 3))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (gy base) (priv-stor locn)
    (l l-peer x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-peer))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l) (undisclosed l-peer))
  (rule gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-peer) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b
            (hash (exp (gen) (mul l l-peer)) (exp gy x)))))))
  (label 1)
  (parent 0)
  (unrealized (0 0) (0 1) (0 3))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval) (gy base)
    (priv-stor locn) (l-peer x l rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-peer))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l) (undisclosed l-peer))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-peer) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b
            (hash (exp (gen) (mul l-peer l)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 2)
  (parent 1)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 3)
  (parent 2)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x)))))))
  (label 4)
  (parent 3)
  (unrealized (3 0) (3 1) (3 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener (hash (exp (gen) (mul l l-0)) (exp gy x)))
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x)))))
  (label 5)
  (parent 3)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x)))))))
  (label 6)
  (parent 4)
  (unrealized (3 1) (3 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 7)
  (parent 4)
  (unrealized (3 1) (3 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener (cat (exp (gen) (mul l l-0)) (exp gy x)))
    (hash (exp (gen) (mul l l-0)) (exp gy x)) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x)))))
  (label 8)
  (parent 5)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 9)
  (parent 6)
  (unrealized (2 2))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 10)
  (parent 6)
  (unrealized (3 2) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 11)
  (parent 7)
  (unrealized (3 2) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 12)
  (parent 7)
  (unrealized (3 2) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 13)
  (parent 7)
  (unrealized (3 2) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 14)
  (parent 8)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 15)
  (parent 8)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 16)
  (parent 9)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 17)
  (parent 9)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 0 init 3) (exp (gen) x-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 18)
  (parent 9)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (l l-0) (l-peer l) (x x) (gy (exp (gen) y))
        (na na) (nb nb) (priv-stor priv-stor-0))))
  (origs (na (0 2)) (l-0 (3 1)) (pt-2 (3 1)) (l (1 1)) (pt-0 (1 1))
    (nb (2 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 3 ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 19)
  (parent 9)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 1 ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 20)
  (parent 9)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 21)
  (parent 9)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp gy (mul x (rec y) (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec y) (rec w))) w))
    (exp gy (mul x (rec y))) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp gy (mul x (rec y) (rec w))) w))
      (send (cat (exp gy (mul x (rec y) (rec w))) w))))
  (label 22)
  (parent 9)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-1))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-1) (pv b l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 23)
  (parent 10)
  (unrealized (3 1) (3 2) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul (rec y) x))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul (rec y) x))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 24)
  (parent 10)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 25)
  (parent 10)
  (unrealized (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 26)
  (parent 10)
  (unrealized (2 2))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-1))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-1) (pv b l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y-0)))))))
  (label 27)
  (parent 10)
  (unrealized (3 1) (3 2) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 28)
  (parent 10)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 29)
  (parent 10)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 30)
  (parent 11)
  (unrealized (3 2) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 31)
  (parent 16)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0)))))))
  (label 32)
  (parent 16)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 33)
  (parent 16)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 34)
  (parent 17)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 35)
  (parent 17)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 36)
  (parent 17)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 37)
  (parent 19)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 38)
  (parent 19)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 39)
  (parent 19)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 40)
  (parent 20)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 41)
  (parent 20)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 42)
  (parent 20)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 43)
  (parent 21)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 44)
  (parent 21)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 45)
  (parent 21)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 46)
  (parent 22)
  (unrealized (0 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 47)
  (parent 22)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) y w))))
    (gen) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 48)
  (parent 22)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 49)
  (parent 22)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 50)
  (parent 22)
  (unrealized (0 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 3 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 51)
  (parent 22)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 52)
  (parent 22)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gy gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 53)
  (parent 23)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gy gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 54)
  (parent 23)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 55)
  (parent 23)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 56)
  (parent 26)
  (unrealized (0 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 57)
  (parent 26)
  (unrealized (0 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 58)
  (parent 26)
  (seen 18)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 4 ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 59)
  (parent 26)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 60)
  (parent 26)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 61)
  (parent 26)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp gy (mul x (rec y) (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec y) (rec w))) w))
    (exp gy (mul x (rec y))) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp gy (mul x (rec y) (rec w))) w))
      (send (cat (exp gy (mul x (rec y) (rec w))) w))))
  (label 62)
  (parent 26)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gy ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y-0)))))))
  (label 63)
  (parent 27)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gy ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 64)
  (parent 27)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 65)
  (parent 27)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 66)
  (parent 30)
  (unrealized (3 2) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 67)
  (parent 30)
  (unrealized (3 2) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 68)
  (parent 31)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 69)
  (parent 31)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 70)
  (parent 31)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 71)
  (parent 32)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 72)
  (parent 32)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 73)
  (parent 32)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 74)
  (parent 33)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 75)
  (parent 33)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 76)
  (parent 33)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 77)
  (parent 34)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 78)
  (parent 34)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 79)
  (parent 34)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 80)
  (parent 35)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 81)
  (parent 35)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 82)
  (parent 35)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 83)
  (parent 36)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 84)
  (parent 36)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 85)
  (parent 36)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (6 0)) ((4 2) (2 2))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv self-0 l-1))))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 86)
  (parent 43)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 87)
  (parent 46)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 88)
  (parent 46)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 89)
  (parent 46)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 90)
  (parent 47)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (l l-0) (l-peer l) (x x)
        (gy (exp (gen) (mul y w))) (na na) (nb nb)
        (priv-stor priv-stor-0))))
  (origs (na (0 2)) (l-0 (3 1)) (pt-2 (3 1)) (l (1 1)) (pt-0 (1 1))
    (nb (2 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) x))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 91)
  (parent 48)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 92)
  (parent 48)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0))))))
  (label 93)
  (parent 49)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 94)
  (parent 49)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 95)
  (parent 50)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0)))))))
  (label 96)
  (parent 50)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 97)
  (parent 50)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l))))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul x (rec l))))
      (send (cat (exp (gen) l) (mul x (rec l))))))
  (label 98)
  (parent 51)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 99)
  (parent 51)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (4 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 100)
  (parent 52)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 101)
  (parent 52)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 102)
  (parent 56)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0)))))))
  (label 103)
  (parent 56)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 104)
  (parent 56)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 105)
  (parent 57)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 106)
  (parent 57)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 107)
  (parent 57)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 108)
  (parent 59)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 109)
  (parent 59)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 110)
  (parent 59)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 111)
  (parent 60)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 112)
  (parent 60)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 113)
  (parent 60)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 114)
  (parent 61)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 115)
  (parent 61)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 116)
  (parent 61)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 117)
  (parent 62)
  (unrealized (0 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 118)
  (parent 62)
  (seen 47)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) y w))))
    (gen) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 119)
  (parent 62)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 120)
  (parent 62)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 121)
  (parent 62)
  (unrealized (0 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 4 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 122)
  (parent 62)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 123)
  (parent 62)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (6 0)) ((4 2) (2 2))
    ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 124)
  (parent 86)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (6 0)) ((4 2) (2 2))
    ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 125)
  (parent 86)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 126)
  (parent 87)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 127)
  (parent 87)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 128)
  (parent 88)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 129)
  (parent 88)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (4 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))))
  (label 130)
  (parent 89)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 131)
  (parent 89)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 132)
  (parent 92)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x w (rec l-0))))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w (rec l-0)))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 133)
  (parent 94)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y)))))))
  (label 134)
  (parent 95)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 135)
  (parent 95)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y)))))))
  (label 136)
  (parent 96)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 137)
  (parent 96)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0)))))
  (label 138)
  (parent 97)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 139)
  (parent 97)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x l rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul x (rec l) w)))
      (send (cat (exp (gen) l) (mul x (rec l) w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 140)
  (parent 99)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (4 0) (6 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 141)
  (parent 101)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 142)
  (parent 102)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 143)
  (parent 102)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 144)
  (parent 102)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 145)
  (parent 103)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 146)
  (parent 103)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 147)
  (parent 103)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 148)
  (parent 104)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 149)
  (parent 104)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 150)
  (parent 104)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 151)
  (parent 105)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 152)
  (parent 105)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 153)
  (parent 105)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 154)
  (parent 106)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 155)
  (parent 106)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 156)
  (parent 106)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 157)
  (parent 107)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 158)
  (parent 107)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 159)
  (parent 107)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (7 0))
    ((5 2) (2 2)) ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((5 1) (7 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (6 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv self-0 l-1))))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 160)
  (parent 114)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 161)
  (parent 117)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 162)
  (parent 117)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 163)
  (parent 117)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) x))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 164)
  (parent 119)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 165)
  (parent 119)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0))))))
  (label 166)
  (parent 120)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 167)
  (parent 120)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 168)
  (parent 121)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0)))))))
  (label 169)
  (parent 121)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 170)
  (parent 121)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l))))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul x (rec l))))
      (send (cat (exp (gen) l) (mul x (rec l))))))
  (label 171)
  (parent 122)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 172)
  (parent 122)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 173)
  (parent 123)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 174)
  (parent 123)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 175)
  (parent 127)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 176)
  (parent 129)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 177)
  (parent 131)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 178)
  (parent 132)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 179)
  (parent 132)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 180)
  (parent 132)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 181)
  (parent 132)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 182)
  (parent 133)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 183)
  (parent 133)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 184)
  (parent 133)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 185)
  (parent 133)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 186)
  (parent 135)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 187)
  (parent 137)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx) (w expt)
    (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 188)
  (parent 139)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 189)
  (parent 140)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l-0) (w (mul (rec x) l-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 190)
  (parent 140)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 191)
  (parent 140)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 192)
  (parent 140)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-1) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 193)
  (parent 141)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1)))))
  (label 194)
  (parent 141)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-1) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 195)
  (parent 141)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 5 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1)))))
  (label 196)
  (parent 141)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 y x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-2))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-1 l-2)))))
  (label 197)
  (parent 141)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (7 0))
    ((5 2) (2 2)) ((6 1) (0 3)) ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((5 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 198)
  (parent 160)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (7 0))
    ((5 2) (2 2)) ((6 1) (0 3)) ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((5 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 199)
  (parent 160)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 200)
  (parent 161)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 201)
  (parent 161)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 202)
  (parent 162)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 203)
  (parent 162)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))))
  (label 204)
  (parent 163)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 205)
  (parent 163)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 206)
  (parent 165)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x w (rec l-0))))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w (rec l-0)))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 207)
  (parent 167)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y)))))))
  (label 208)
  (parent 168)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 209)
  (parent 168)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y)))))))
  (label 210)
  (parent 169)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 211)
  (parent 169)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0)))))
  (label 212)
  (parent 170)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 213)
  (parent 170)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x l rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul x (rec l) w)))
      (send (cat (exp (gen) l) (mul x (rec l) w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 214)
  (parent 172)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 215)
  (parent 174)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 216)
  (parent 175)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 217)
  (parent 175)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 218)
  (parent 175)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 219)
  (parent 175)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 220)
  (parent 176)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 221)
  (parent 176)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 222)
  (parent 176)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 223)
  (parent 176)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 224)
  (parent 177)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 225)
  (parent 177)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l-1) (x-0 l))
  (precur (4 0) (7 0))
  (gen-st (pv b l-0) (pv self l-1) (pv a l))
  (facts (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 226)
  (parent 177)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 6 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 227)
  (parent 177)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-2))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-0 l-2)))))
  (label 228)
  (parent 177)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 229)
  (parent 178)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 230)
  (parent 179)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (4 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1))))
    ((recv x) (send x)))
  (label 231)
  (parent 180)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 232)
  (parent 182)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 233)
  (parent 186)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 234)
  (parent 186)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 235)
  (parent 186)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 236)
  (parent 186)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 237)
  (parent 187)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 238)
  (parent 187)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 239)
  (parent 187)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 240)
  (parent 187)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 241)
  (parent 188)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 242)
  (parent 188)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l-1) (y-0 l))
  (precur (4 0) (7 0))
  (gen-st (pv b l-0) (pv self l-1) (pv b-0 l))
  (facts (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 243)
  (parent 188)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 6 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 244)
  (parent 188)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-2))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-2)) (send (cat (exp (gen) y-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-0 l-2)))))
  (label 245)
  (parent 188)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0))
    ((4 1) (2 2)) ((5 1) (0 3)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 246)
  (parent 191)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1 l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1))))
    ((recv x) (send x)))
  (label 247)
  (parent 196)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 248)
  (parent 201)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 249)
  (parent 203)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 250)
  (parent 205)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 251)
  (parent 206)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 252)
  (parent 206)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 253)
  (parent 206)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 254)
  (parent 206)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 255)
  (parent 207)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) l-0)) (l-1 l-0)) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 256)
  (parent 207)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 257)
  (parent 207)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 258)
  (parent 207)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 259)
  (parent 209)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 260)
  (parent 211)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx) (w expt)
    (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 261)
  (parent 213)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 262)
  (parent 214)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l-0) (w (mul (rec x) l-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 263)
  (parent 214)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 264)
  (parent 214)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 265)
  (parent 214)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-1) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 266)
  (parent 215)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1)))))
  (label 267)
  (parent 215)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-1) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 268)
  (parent 215)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 6 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1)))))
  (label 269)
  (parent 215)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 y x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-2))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-1 l-2)))))
  (label 270)
  (parent 215)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 271)
  (parent 248)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 272)
  (parent 248)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 273)
  (parent 248)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 274)
  (parent 248)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 275)
  (parent 249)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 276)
  (parent 249)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 277)
  (parent 249)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 278)
  (parent 249)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 279)
  (parent 250)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 280)
  (parent 250)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l-1) (x-0 l))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-0) (pv self l-1) (pv a l))
  (facts (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 281)
  (parent 250)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 282)
  (parent 250)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-2))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)) ((9 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-0 l-2)))))
  (label 283)
  (parent 250)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 284)
  (parent 251)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 285)
  (parent 252)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (5 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1))))
    ((recv x) (send x)))
  (label 286)
  (parent 253)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 287)
  (parent 255)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 288)
  (parent 259)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 289)
  (parent 259)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 290)
  (parent 259)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 291)
  (parent 259)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 292)
  (parent 260)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 293)
  (parent 260)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 294)
  (parent 260)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 295)
  (parent 260)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 296)
  (parent 261)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 297)
  (parent 261)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l-1) (y-0 l))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-0) (pv self l-1) (pv b-0 l))
  (facts (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 298)
  (parent 261)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 299)
  (parent 261)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-2))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)) ((9 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-2)) (send (cat (exp (gen) y-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-0 l-2)))))
  (label 300)
  (parent 261)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 301)
  (parent 264)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1 l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1))))
    ((recv x) (send x)))
  (label 302)
  (parent 269)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (l x rndx) (gb gy base) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a l))
      (recv (sig (body b gb (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv (cat gy (enc na nb a b (hash (exp gb l) (exp gy x)))))
      (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x l))
    (gen-st (pv a l))
    (fn-of ("principal-of" (ltxa a) (ltxb b))
      ("ltx-of" (a ltxa) (b ltxb))))
  (defrole resp
    (vars (l y rndx) (ga gx base) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b l))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b gx))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp ga l) (exp gx y))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y l))
    (gen-st (pv b l))
    (fn-of ("principal-of" (ltxa a) (ltxb b))
      ("ltx-of" (a ltxa) (b ltxb))))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l)
    (fn-of ("principal-of" (l self)) ("ltx-of" (self l))))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l))
    (fn-of ("principal-of" (l self)) ("ltx-of" (self l))))
  (defrule fact-resp-neq0
    (forall ((z strd))
      (implies
        (and (p "resp" z (idx 3)) (p "resp" "gx" z (gen)))
        (false))))
  (defrule fact-init-neq0
    (forall ((z strd))
      (implies
        (and (p "init" z (idx 4)) (p "init" "gy" z (gen)))
        (false))))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (a name) (l rndx))
      (implies
        (and (p "init" z (idx 1)) (p "init" "l" z l) (p "init" "a" z a))
        (gen-st (pv a l)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (b name) (l rndx))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "l" z l) (p "resp" "b" z b))
        (gen-st (pv b l)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (self name) (l rndx))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (gy base) (priv-stor locn)
    (l l-peer x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-peer))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x l))
  (facts (neq a b) (undisclosed l-peer))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-peer) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b
            (hash (exp (gen) (mul l l-peer)) (exp gy x)))))))
  (label 303)
  (unrealized (0 1) (0 3))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (gy base) (priv-stor locn)
    (l l-peer x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-peer))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-peer))
  (rule gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-peer) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b
            (hash (exp (gen) (mul l l-peer)) (exp gy x)))))))
  (label 304)
  (parent 303)
  (unrealized (0 0) (0 1) (0 3))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval) (gy base)
    (priv-stor locn) (l-peer x l rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-peer))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-peer))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-peer) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b
            (hash (exp (gen) (mul l-peer l)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 305)
  (parent 304)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 306)
  (parent 305)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x)))))))
  (label 307)
  (parent 306)
  (unrealized (3 0) (3 1) (3 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener (hash (exp (gen) (mul l l-0)) (exp gy x)))
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x)))))
  (label 308)
  (parent 306)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x)))))))
  (label 309)
  (parent 307)
  (unrealized (3 1) (3 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 310)
  (parent 307)
  (unrealized (3 1) (3 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener (cat (exp (gen) (mul l l-0)) (exp gy x)))
    (hash (exp (gen) (mul l l-0)) (exp gy x)) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x)))))
  (label 311)
  (parent 308)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 312)
  (parent 309)
  (unrealized (2 2))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 313)
  (parent 309)
  (unrealized (3 2) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 314)
  (parent 310)
  (unrealized (3 2) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 315)
  (parent 310)
  (unrealized (3 2) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 316)
  (parent 310)
  (unrealized (3 2) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 317)
  (parent 311)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 318)
  (parent 311)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 319)
  (parent 312)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 320)
  (parent 312)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 0 init 3) (exp (gen) x-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 321)
  (parent 312)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (l l-0) (l-peer l) (x x) (gy (exp (gen) y))
        (na na) (nb nb) (priv-stor priv-stor-0))))
  (origs (na (0 2)) (l-0 (3 1)) (pt-2 (3 1)) (l (1 1)) (pt-0 (1 1))
    (nb (2 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 3 ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 322)
  (parent 312)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 1 ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 323)
  (parent 312)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 324)
  (parent 312)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp gy (mul x (rec y) (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec y) (rec w))) w))
    (exp gy (mul x (rec y))) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp gy (mul x (rec y) (rec w))) w))
      (send (cat (exp gy (mul x (rec y) (rec w))) w))))
  (label 325)
  (parent 312)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-1))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-1) (pv b l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 326)
  (parent 313)
  (unrealized (3 1) (3 2) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul (rec y) x))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul (rec y) x))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 327)
  (parent 313)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 328)
  (parent 313)
  (unrealized (3 2) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 329)
  (parent 313)
  (unrealized (2 2))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-1))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-1) (pv b l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y-0)))))))
  (label 330)
  (parent 313)
  (unrealized (3 1) (3 2) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 331)
  (parent 313)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 332)
  (parent 313)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 333)
  (parent 314)
  (unrealized (3 2) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 334)
  (parent 316)
  (unrealized (3 2) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 335)
  (parent 318)
  (unrealized (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 336)
  (parent 319)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0)))))))
  (label 337)
  (parent 319)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 338)
  (parent 319)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 339)
  (parent 320)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 340)
  (parent 320)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 341)
  (parent 320)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 342)
  (parent 322)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 343)
  (parent 322)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 344)
  (parent 322)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 345)
  (parent 323)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 346)
  (parent 323)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 347)
  (parent 323)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 348)
  (parent 324)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 349)
  (parent 324)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 350)
  (parent 324)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 351)
  (parent 325)
  (unrealized (0 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 352)
  (parent 325)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) y w))))
    (gen) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 353)
  (parent 325)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 354)
  (parent 325)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 355)
  (parent 325)
  (unrealized (0 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 3 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 356)
  (parent 325)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 357)
  (parent 325)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gy gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 358)
  (parent 326)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gy gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 359)
  (parent 326)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 360)
  (parent 326)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 361)
  (parent 328)
  (unrealized (3 2))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 362)
  (parent 329)
  (unrealized (0 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 363)
  (parent 329)
  (unrealized (0 3) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 364)
  (parent 329)
  (seen 321)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 4 ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 365)
  (parent 329)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 366)
  (parent 329)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 367)
  (parent 329)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp gy (mul x (rec y) (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec y) (rec w))) w))
    (exp gy (mul x (rec y))) (2 2))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp gy (mul x (rec y) (rec w))) w))
      (send (cat (exp gy (mul x (rec y) (rec w))) w))))
  (label 368)
  (parent 329)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gy ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y-0)))))))
  (label 369)
  (parent 330)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gy ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 370)
  (parent 330)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 371)
  (parent 330)
  (unrealized (3 1) (3 2) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 372)
  (parent 333)
  (unrealized (3 2) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 373)
  (parent 333)
  (unrealized (3 2) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 374)
  (parent 334)
  (unrealized (3 2) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 375)
  (parent 334)
  (unrealized (3 2) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 376)
  (parent 335)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 377)
  (parent 335)
  (unrealized (4 0) (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 378)
  (parent 336)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 379)
  (parent 336)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 380)
  (parent 336)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 381)
  (parent 337)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 382)
  (parent 337)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 3) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 383)
  (parent 337)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 384)
  (parent 338)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 385)
  (parent 338)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 3) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 386)
  (parent 338)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 387)
  (parent 339)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 388)
  (parent 339)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (4 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 389)
  (parent 339)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 390)
  (parent 340)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 391)
  (parent 340)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 1) (4 0)) ((3 2) (2 1)) ((4 2) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 392)
  (parent 340)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 393)
  (parent 341)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 394)
  (parent 341)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 2) (2 2)) ((5 1) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 395)
  (parent 341)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv self l-0))))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 396)
  (parent 342)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (6 0)) ((4 2) (2 2))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv self-0 l-1))))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 397)
  (parent 348)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 398)
  (parent 351)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 399)
  (parent 351)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 400)
  (parent 351)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 401)
  (parent 352)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (l l-0) (l-peer l) (x x)
        (gy (exp (gen) (mul y w))) (na na) (nb nb)
        (priv-stor priv-stor-0))))
  (origs (na (0 2)) (l-0 (3 1)) (pt-2 (3 1)) (l (1 1)) (pt-0 (1 1))
    (nb (2 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) x))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 402)
  (parent 353)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 403)
  (parent 353)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0))))))
  (label 404)
  (parent 354)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 405)
  (parent 354)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 406)
  (parent 355)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0)))))))
  (label 407)
  (parent 355)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 408)
  (parent 355)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l))))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul x (rec l))))
      (send (cat (exp (gen) l) (mul x (rec l))))))
  (label 409)
  (parent 356)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 410)
  (parent 356)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (4 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 411)
  (parent 357)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 412)
  (parent 357)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 413)
  (parent 361)
  (unrealized (0 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 414)
  (parent 361)
  (unrealized (0 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 415)
  (parent 361)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 ltx-gen 3) (exp (gen) l-1) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 416)
  (parent 361)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 1) (4 0))
    ((5 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 6 ltx-gen 3) (exp (gen) l-1) (3 2))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 417)
  (parent 361)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 418)
  (parent 361)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp gy (mul x (rec y)))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp gy (mul x (rec y) (rec w))) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec y) (rec w))) w))
    (exp gy (mul x (rec y))) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp gy (mul x (rec y) (rec w))) w))
      (send (cat (exp gy (mul x (rec y) (rec w))) w))))
  (label 419)
  (parent 361)
  (unrealized (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 420)
  (parent 362)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 3) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0)))))))
  (label 421)
  (parent 362)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 422)
  (parent 362)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 423)
  (parent 363)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 424)
  (parent 363)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 425)
  (parent 363)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 426)
  (parent 365)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 427)
  (parent 365)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 428)
  (parent 365)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 429)
  (parent 366)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 430)
  (parent 366)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 431)
  (parent 366)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 432)
  (parent 367)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 433)
  (parent 367)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 434)
  (parent 367)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 435)
  (parent 368)
  (unrealized (0 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 436)
  (parent 368)
  (seen 352)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) y w))))
    (gen) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 437)
  (parent 368)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 438)
  (parent 368)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 439)
  (parent 368)
  (unrealized (0 3) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 4 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 440)
  (parent 368)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 441)
  (parent 368)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 442)
  (parent 373)
  (unrealized (3 2) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 443)
  (parent 374)
  (unrealized (3 2) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 444)
  (parent 376)
  (unrealized (0 3) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 445)
  (parent 376)
  (unrealized (0 3) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (rec x)))) (gen)
    (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 446)
  (parent 376)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 447)
  (parent 376)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 1) (5 0)) ((6 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 448)
  (parent 376)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 449)
  (parent 376)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp gy (mul x (rec w))) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec w))) w)) (exp gy x) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp gy (mul x (rec w))) w))
      (send (cat (exp gy (mul x (rec w))) w))))
  (label 450)
  (parent 376)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp gy x)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-1)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-1)) (exp gy x))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 451)
  (parent 377)
  (unrealized (4 0) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 452)
  (parent 377)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 453)
  (parent 377)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 454)
  (parent 377)
  (unrealized (3 0) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gy ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp gy x)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-1 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-1)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-1)) (exp gy x))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-1 (rec y))))
      (send (cat (exp (gen) y) (mul l-1 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y)))))))
  (label 455)
  (parent 377)
  (unrealized (4 0) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 456)
  (parent 377)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 457)
  (parent 377)
  (unrealized (4 0) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)) ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 458)
  (parent 396)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)) ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 459)
  (parent 396)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (6 0)) ((4 2) (2 2))
    ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 460)
  (parent 397)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (6 0)) ((4 2) (2 2))
    ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 461)
  (parent 397)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 462)
  (parent 398)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 463)
  (parent 398)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 464)
  (parent 399)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 465)
  (parent 399)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (4 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))))
  (label 466)
  (parent 400)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 467)
  (parent 400)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 468)
  (parent 403)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x w (rec l-0))))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w (rec l-0)))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 469)
  (parent 405)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y)))))))
  (label 470)
  (parent 406)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 471)
  (parent 406)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y)))))))
  (label 472)
  (parent 407)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 473)
  (parent 407)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0)))))
  (label 474)
  (parent 408)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 475)
  (parent 408)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x l rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul x (rec l) w)))
      (send (cat (exp (gen) l) (mul x (rec l) w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 476)
  (parent 410)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (4 0) (6 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 477)
  (parent 412)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y-0)))))))
  (label 478)
  (parent 413)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 479)
  (parent 413)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 3) (3 2)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 480)
  (parent 413)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 481)
  (parent 414)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 482)
  (parent 414)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 2) (3 2)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 483)
  (parent 414)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 484)
  (parent 415)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 485)
  (parent 416)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 486)
  (parent 416)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 487)
  (parent 416)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 1) (4 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-0))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0))))
  (label 488)
  (parent 417)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) l-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 1) (4 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-0)) y))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-0)) y))
      (send (cat (exp (gen) (mul (rec x) l-0)) y))))
  (label 489)
  (parent 417)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul y l-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 1) (4 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-0)) x))
    (exp (gen) (mul (rec x) y l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-0)) x))
      (send (cat (exp (gen) (mul y l-0)) x))))
  (label 490)
  (parent 417)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 2) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 491)
  (parent 418)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 2) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 492)
  (parent 418)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 2) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 493)
  (parent 418)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 494)
  (parent 419)
  (unrealized (0 3) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y l rndx) (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 495)
  (parent 419)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) y w))))
    (gen) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 496)
  (parent 419)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 497)
  (parent 419)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 498)
  (parent 419)
  (unrealized (0 3) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 499)
  (parent 419)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 500)
  (parent 419)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 501)
  (parent 420)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 502)
  (parent 420)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 503)
  (parent 420)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 504)
  (parent 421)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 505)
  (parent 421)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 3) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 506)
  (parent 421)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 507)
  (parent 422)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 508)
  (parent 422)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 3) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 509)
  (parent 422)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 510)
  (parent 423)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 511)
  (parent 423)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 512)
  (parent 423)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 513)
  (parent 424)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 514)
  (parent 424)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (3 0))
    ((5 2) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 515)
  (parent 424)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 516)
  (parent 425)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 517)
  (parent 425)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 2) (2 2))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 518)
  (parent 425)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv self l-0))))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 519)
  (parent 426)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (7 0))
    ((5 2) (2 2)) ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((5 1) (7 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (6 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv self-0 l-1))))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 520)
  (parent 432)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 521)
  (parent 435)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 522)
  (parent 435)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 523)
  (parent 435)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) x))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 524)
  (parent 437)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 525)
  (parent 437)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0))))))
  (label 526)
  (parent 438)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 527)
  (parent 438)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 528)
  (parent 439)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0)))))))
  (label 529)
  (parent 439)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 530)
  (parent 439)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l))))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul x (rec l))))
      (send (cat (exp (gen) l) (mul x (rec l))))))
  (label 531)
  (parent 440)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y l rndx)
    (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 532)
  (parent 440)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 533)
  (parent 441)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 534)
  (parent 441)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 535)
  (parent 442)
  (unrealized (3 2))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 536)
  (parent 442)
  (unrealized (3 2) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 537)
  (parent 443)
  (unrealized (3 2))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 538)
  (parent 443)
  (unrealized (3 2) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (7 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 539)
  (parent 444)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 540)
  (parent 444)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x l l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 3) (4 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 541)
  (parent 444)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (7 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 542)
  (parent 445)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 543)
  (parent 445)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 2) (4 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 544)
  (parent 445)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w))) w))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w))) w))
    (exp (gen) (rec x)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul (rec x) (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) (rec w))) w))))
  (label 545)
  (parent 446)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) l-0))
    (exp (gen) (mul (rec x) l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0))))
  (label 546)
  (parent 447)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) x))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) x))
    (exp (gen) (mul (rec x) l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) x)) (send (cat (exp (gen) l-0) x))))
  (label 547)
  (parent 447)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) l-0))
    (exp (gen) (mul (rec x) l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0))))
  (label 548)
  (parent 448)
  (seen 703)
  (unrealized (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) x))
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) x))
    (exp (gen) (mul (rec x) l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) x)) (send (cat (exp (gen) l-0) x))))
  (label 549)
  (parent 448)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (8 0)) ((7 2) (4 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) l-1))
    (exp (gen) (mul (rec x) l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1))))
  (label 550)
  (parent 449)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) x))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 2) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) x))
    (exp (gen) (mul (rec x) l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) x)) (send (cat (exp (gen) l-1) x))))
  (label 551)
  (parent 449)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 552)
  (parent 450)
  (unrealized (0 3) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 553)
  (parent 450)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) w))))
    (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) w))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 554)
  (parent 450)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 555)
  (parent 450)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 556)
  (parent 450)
  (unrealized (0 3) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 557)
  (parent 450)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 558)
  (parent 450)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy gb base)
    (priv-stor priv-stor-0 locn) (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 559)
  (parent 451)
  (unrealized (4 0) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy gb base)
    (priv-stor priv-stor-0 locn) (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 560)
  (parent 451)
  (unrealized (4 0) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gy gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 561)
  (parent 451)
  (unrealized (4 0) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 562)
  (parent 455)
  (unrealized (4 0) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 563)
  (parent 455)
  (unrealized (4 0) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x l l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 564)
  (parent 455)
  (unrealized (4 0) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 565)
  (parent 456)
  (unrealized (0 3) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 566)
  (parent 456)
  (unrealized (0 3) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (rec x)))) (gen)
    (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 567)
  (parent 456)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 568)
  (parent 456)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 569)
  (parent 456)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 570)
  (parent 456)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gy base) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp gy x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp gy (mul x (rec w))) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec w))) w)) (exp gy x) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp gy x)))
      (send (hash (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp gy x)))
      (send (cat (exp (gen) (mul l l-0)) (exp gy x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp gy (mul x (rec w))) w))
      (send (cat (exp gy (mul x (rec w))) w))))
  (label 571)
  (parent 456)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 572)
  (parent 463)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 573)
  (parent 465)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 574)
  (parent 467)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 575)
  (parent 468)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 576)
  (parent 468)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 577)
  (parent 468)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 578)
  (parent 468)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 579)
  (parent 469)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 580)
  (parent 469)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 581)
  (parent 469)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 1) (0 3))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 582)
  (parent 469)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 583)
  (parent 471)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 584)
  (parent 473)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx) (w expt)
    (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 585)
  (parent 475)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 586)
  (parent 476)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l-0) (w (mul (rec x) l-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 587)
  (parent 476)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 588)
  (parent 476)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 589)
  (parent 476)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-1) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 590)
  (parent 477)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1)))))
  (label 591)
  (parent 477)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 592)
  (parent 477)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 5 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1)))))
  (label 593)
  (parent 477)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 y x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-2))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-1 l-2)))))
  (label 594)
  (parent 477)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 3) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 595)
  (parent 478)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 3) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 596)
  (parent 478)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 3) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 597)
  (parent 478)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 3) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 598)
  (parent 479)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 3) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 599)
  (parent 479)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 3) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 600)
  (parent 479)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 3) (3 2)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 601)
  (parent 480)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 3) (3 2)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 602)
  (parent 480)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 3) (3 2)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 603)
  (parent 480)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 2) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 604)
  (parent 481)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 2) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 605)
  (parent 481)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 2) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 606)
  (parent 481)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 2) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 607)
  (parent 482)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 2) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 608)
  (parent 482)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 2) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 609)
  (parent 482)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 2) (3 2)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 610)
  (parent 483)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 2) (3 2)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 611)
  (parent 483)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 2) (3 2)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 612)
  (parent 483)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 613)
  (parent 484)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 1) (4 0))
    ((5 2) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0)))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 614)
  (parent 488)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 1) (4 0))
    ((5 2) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)) ((5 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0)))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 615)
  (parent 488)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (8 0)) ((6 2) (3 2)) ((7 1) (0 3)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((6 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-5 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 616)
  (parent 491)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 617)
  (parent 494)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 618)
  (parent 494)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 619)
  (parent 494)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y l rndx) (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 620)
  (parent 495)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) x))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 621)
  (parent 496)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx) (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) w)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 622)
  (parent 496)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) (mul x (rec l))))
      (send (cat (exp (gen) l) (mul x (rec l)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 623)
  (parent 497)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 624)
  (parent 497)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y-0)))))))
  (label 625)
  (parent 498)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l rndx) (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0)))))))
  (label 626)
  (parent 498)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 627)
  (parent 498)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0))))))
  (label 628)
  (parent 499)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-0 (rec w-0))) w-0))))
  (label 629)
  (parent 499)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l-1) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 630)
  (parent 500)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w l-1))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 631)
  (parent 500)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 632)
  (parent 519)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-0))
      (send (cat (exp (gen) (mul (rec x) y)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 633)
  (parent 519)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (7 0))
    ((5 2) (2 2)) ((6 1) (0 3)) ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((5 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 634)
  (parent 520)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (7 0))
    ((5 2) (2 2)) ((6 1) (0 3)) ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((5 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 635)
  (parent 520)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 636)
  (parent 521)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 637)
  (parent 521)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 638)
  (parent 522)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 639)
  (parent 522)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))))
  (label 640)
  (parent 523)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 641)
  (parent 523)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 642)
  (parent 525)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x w (rec l-0))))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w (rec l-0)))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 643)
  (parent 527)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y)))))))
  (label 644)
  (parent 528)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 645)
  (parent 528)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y)))))))
  (label 646)
  (parent 529)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (y-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 647)
  (parent 529)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0)))))
  (label 648)
  (parent 530)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 649)
  (parent 530)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x l rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul x (rec l) w)))
      (send (cat (exp (gen) l) (mul x (rec l) w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 650)
  (parent 532)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 651)
  (parent 534)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0)))))))
  (label 652)
  (parent 535)
  (unrealized (0 3) (9 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 653)
  (parent 535)
  (unrealized (0 3) (9 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 654)
  (parent 535)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 655)
  (parent 535)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (7 0))
    ((8 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 656)
  (parent 535)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (5 0))
    ((8 2) (3 2)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((8 1) (3 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 9 ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 657)
  (parent 535)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 658)
  (parent 535)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp gy (mul x (rec y) (rec w))) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec y) (rec w))) w))
    (exp gy (mul x (rec y))) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp gy (mul x (rec y) (rec w))) w))
      (send (cat (exp gy (mul x (rec y) (rec w))) w))))
  (label 659)
  (parent 535)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-2))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-2))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-2))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-2))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-2)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-2)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-2)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 660)
  (parent 536)
  (unrealized (3 2) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gy gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-2))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-2))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-2))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-2)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-2))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-2)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-2)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-2)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-2)) l))
      (send (cat (exp (gen) (mul l-0 l-2)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 661)
  (parent 536)
  (unrealized (3 2) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 662)
  (parent 536)
  (unrealized (3 2) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul (rec y) x)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul (rec y) x))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 663)
  (parent 536)
  (unrealized (3 2) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 664)
  (parent 536)
  (unrealized (3 2))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 665)
  (parent 536)
  (unrealized (2 2) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 666)
  (parent 536)
  (unrealized (3 2) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 667)
  (parent 536)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0)))))))
  (label 668)
  (parent 537)
  (unrealized (0 3) (9 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 669)
  (parent 537)
  (unrealized (0 3) (9 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l)))
  (label 670)
  (parent 537)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 671)
  (parent 537)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 672)
  (parent 537)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (7 0))
    ((8 2) (3 2)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((8 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 9 ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 673)
  (parent 537)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 674)
  (parent 537)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp gy (mul x (rec y) (rec w))) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec y) (rec w))) w))
    (exp gy (mul x (rec y))) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp gy (mul x (rec y) (rec w))) w))
      (send (cat (exp gy (mul x (rec y) (rec w))) w))))
  (label 675)
  (parent 537)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-2))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-2))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-2))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-2)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-2))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-2)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-2)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-2)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-2)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-2)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0)))))))
  (label 676)
  (parent 538)
  (unrealized (3 2) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gy gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-2))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-2))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-2))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-2)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-2))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-2)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-2)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-2)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-2)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-2)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) l-0)) (send (cat (exp (gen) l-2) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 677)
  (parent 538)
  (unrealized (3 2) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 678)
  (parent 538)
  (unrealized (3 2) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul (rec y) x)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul (rec y) x))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 679)
  (parent 538)
  (unrealized (3 2) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 680)
  (parent 538)
  (unrealized (3 2))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 681)
  (parent 538)
  (unrealized (2 2) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 682)
  (parent 538)
  (unrealized (3 2) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 683)
  (parent 538)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 3) (4 0))
    ((7 3) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 684)
  (parent 539)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 3) (4 0))
    ((7 3) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 685)
  (parent 539)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 3) (4 0))
    ((7 3) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 686)
  (parent 540)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 3) (4 0))
    ((7 3) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 687)
  (parent 540)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x l l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 3) (4 0)) ((7 3) (9 0))
    ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 688)
  (parent 541)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x l l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 3) (4 0)) ((7 3) (9 0))
    ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 689)
  (parent 541)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (rec x)) x-0))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 2) (4 0))
    ((7 2) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) x-0))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (rec x)) x-0))
      (send (cat (exp (gen) (rec x)) x-0))))
  (label 690)
  (parent 542)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) x))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 2) (4 0))
    ((7 2) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) x-0) x))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x))))
  (label 691)
  (parent 542)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (rec x)) x-0))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 2) (4 0))
    ((7 2) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) x-0))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (rec x)) x-0))
      (send (cat (exp (gen) (rec x)) x-0))))
  (label 692)
  (parent 543)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) x))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 2) (4 0))
    ((7 2) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) x-0) x))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x))))
  (label 693)
  (parent 543)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) x-0))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 2) (4 0)) ((7 2) (9 0))
    ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) x-0))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) (rec x)) x-0))
      (send (cat (exp (gen) (rec x)) x-0))))
  (label 694)
  (parent 544)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) x))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 2) (4 0)) ((7 2) (9 0))
    ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) x-0) x))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x))))
  (label 695)
  (parent 544)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 696)
  (parent 545)
  (unrealized (0 3) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 697)
  (parent 545)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 698)
  (parent 545)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb self b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 699)
  (parent 545)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 700)
  (parent 545)
  (unrealized (0 3) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))))
  (label 701)
  (parent 545)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 702)
  (parent 545)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 5 ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-3 (pv self l-0))))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0))))
  (label 703)
  (parent 548)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-1 (cat pt-5 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 704)
  (parent 550)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 705)
  (parent 552)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 706)
  (parent 552)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 707)
  (parent 552)
  (unrealized (0 3))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 708)
  (parent 553)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 709)
  (parent 554)
  (unrealized (4 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 710)
  (parent 554)
  (unrealized (4 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul x x)))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x)))))
  (label 711)
  (parent 554)
  (unrealized (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x l-0)))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x l-0))) (send (cat (gen) (mul x l-0)))))
  (label 712)
  (parent 554)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul x l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (gen) (mul x l-0))) (send (cat (gen) (mul x l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 713)
  (parent 554)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x l-1))) (send (cat (gen) (mul x l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 714)
  (parent 554)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) w))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))))
  (label 715)
  (parent 554)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)) ((8 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0))
  (gen-st (pv self l-1) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y)))))))
  (label 716)
  (parent 555)
  (unrealized (3 0) (6 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0))
  (gen-st (pv self l-1) (pv a l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 717)
  (parent 555)
  (unrealized (3 0) (6 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) x x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) x x)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) x x))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 718)
  (parent 555)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 7 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) x)) (send (cat (exp (gen) l-0) x)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 719)
  (parent 555)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) (mul x (rec l) l-0)))
      (send (cat (exp (gen) l) (mul x (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 720)
  (parent 555)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)) ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 721)
  (parent 555)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))))
  (label 722)
  (parent 555)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 723)
  (parent 556)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 724)
  (parent 556)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx)
    (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 725)
  (parent 556)
  (unrealized (0 3))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) y))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y)))))))
  (label 726)
  (parent 557)
  (unrealized (4 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 727)
  (parent 557)
  (unrealized (4 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) x x)))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) x x)))
      (send (cat (exp (gen) l) (mul (rec l) x x)))))
  (label 728)
  (parent 557)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) x))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) x)) (send (cat (exp (gen) l-0) x))))
  (label 729)
  (parent 557)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul x (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul x (rec l) l-0)))
      (send (cat (exp (gen) l) (mul x (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 730)
  (parent 557)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 731)
  (parent 557)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))))
  (label 732)
  (parent 557)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) (mul x (rec l-2) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (mul x (rec l-2) y)))
      (send (cat (exp (gen) l-2) (mul x (rec l-2) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 733)
  (parent 558)
  (unrealized (4 0) (7 0) (9 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) (mul x (rec l-2) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (mul x (rec l-2) x-0)))
      (send (cat (exp (gen) l-2) (mul x (rec l-2) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 734)
  (parent 558)
  (unrealized (4 0) (7 0) (9 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) x x)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) x x)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) x x))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 735)
  (parent 558)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-2) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 736)
  (parent 558)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (6 0))
  (gen-st (pv self-0 l-1))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-2) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 737)
  (parent 558)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) x))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 8 ltx-gen 3) (exp (gen) l-2) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) x)) (send (cat (exp (gen) l-1) x)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 738)
  (parent 558)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-2)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-2)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 739)
  (parent 558)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w l-1 (rec w-0))) w-0))))
  (label 740)
  (parent 558)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 741)
  (parent 565)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 742)
  (parent 565)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x l l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (4 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 743)
  (parent 565)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 744)
  (parent 566)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 745)
  (parent 566)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (4 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 746)
  (parent 566)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w))) w))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w))) w))
    (exp (gen) (rec x)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) (mul (rec x) (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) (rec w))) w))))
  (label 747)
  (parent 567)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) l-0))
    (exp (gen) (mul (rec x) l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0))))
  (label 748)
  (parent 568)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) x))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) x))
    (exp (gen) (mul (rec x) l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) x)) (send (cat (exp (gen) l-0) x))))
  (label 749)
  (parent 568)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (3 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) l-0))
    (exp (gen) (mul (rec x) l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0))))
  (label 750)
  (parent 569)
  (seen 1062)
  (unrealized (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) x))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (3 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) x))
    (exp (gen) (mul (rec x) l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) x)) (send (cat (exp (gen) l-0) x))))
  (label 751)
  (parent 569)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (9 0))
    ((8 2) (4 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) l-1))
    (exp (gen) (mul (rec x) l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1))))
  (label 752)
  (parent 570)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) x))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) x))
    (exp (gen) (mul (rec x) l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) x)) (send (cat (exp (gen) l-1) x))))
  (label 753)
  (parent 570)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 754)
  (parent 571)
  (unrealized (0 3) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 755)
  (parent 571)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) w))))
    (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) w))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 756)
  (parent 571)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 757)
  (parent 571)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 758)
  (parent 571)
  (unrealized (0 3) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 759)
  (parent 571)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 760)
  (parent 571)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 761)
  (parent 572)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 762)
  (parent 572)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 763)
  (parent 572)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 764)
  (parent 572)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 765)
  (parent 573)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 766)
  (parent 573)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 767)
  (parent 573)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 768)
  (parent 573)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 769)
  (parent 574)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 770)
  (parent 574)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l-1) (x-0 l))
  (precur (4 0) (7 0))
  (gen-st (pv b l-0) (pv self l-1) (pv a l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 771)
  (parent 574)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 6 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 772)
  (parent 574)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-2))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 2) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-0 l-2)))))
  (label 773)
  (parent 574)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 774)
  (parent 575)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 775)
  (parent 576)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (4 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1))))
    ((recv x) (send x)))
  (label 776)
  (parent 577)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 1) (0 3)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 777)
  (parent 579)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 778)
  (parent 583)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 779)
  (parent 583)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 780)
  (parent 583)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (5 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 781)
  (parent 583)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 782)
  (parent 584)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 783)
  (parent 584)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 784)
  (parent 584)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 3) (4 0)) ((6 1) (0 3)) ((7 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((3 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 785)
  (parent 584)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 786)
  (parent 585)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 787)
  (parent 585)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l-1) (y-0 l))
  (precur (4 0) (7 0))
  (gen-st (pv b l-0) (pv self l-1) (pv b-0 l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 788)
  (parent 585)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-1))
  (precur (4 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 6 ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 789)
  (parent 585)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-2))) (priv-stor priv-stor) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2)) ((5 3) (4 0))
    ((6 1) (5 0)) ((7 1) (0 3)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (4 0) (7 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-2)) (send (cat (exp (gen) y-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-0 l-2)))))
  (label 790)
  (parent 585)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((3 2) (4 0))
    ((4 1) (2 2)) ((5 1) (0 3)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 791)
  (parent 588)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 2))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1 l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1))))
    ((recv x) (send x)))
  (label 792)
  (parent 593)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 793)
  (parent 613)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (l l-0) (l-peer l) (x x) (gy (exp (gen) y)) (na na)
        (nb nb) (priv-stor priv-stor))))
  (origs (na (0 2)) (pt-3 (3 1)) (l (2 1)) (pt-2 (2 1)) (l-0 (1 1))
    (pt (1 1))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (8 0)) ((6 2) (3 2)) ((7 1) (0 3)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((6 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 794)
  (parent 616)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (8 0)) ((6 2) (3 2)) ((7 1) (0 3)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((6 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 795)
  (parent 616)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 796)
  (parent 617)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 797)
  (parent 617)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 798)
  (parent 618)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x y l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 799)
  (parent 618)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l-1) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0)))))
  (label 800)
  (parent 619)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 801)
  (parent 619)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y l rndx) (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 802)
  (parent 620)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 803)
  (parent 622)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) (mul x (rec l) w)))
      (send (cat (exp (gen) l) (mul x (rec l) w))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 804)
  (parent 624)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y)))))))
  (label 805)
  (parent 625)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l rndx) (w expt) (y-0 l-0 rndx)
    (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 806)
  (parent 625)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 807)
  (parent 626)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l rndx) (w expt) (y-0 l-0 rndx)
    (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 808)
  (parent 626)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x)) (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 809)
  (parent 627)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 810)
  (parent 627)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x w (rec l-0))))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 3 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w (rec l-0)))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 811)
  (parent 629)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-1) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 812)
  (parent 631)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 813)
  (parent 637)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 814)
  (parent 639)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 815)
  (parent 641)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 816)
  (parent 642)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 817)
  (parent 642)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 818)
  (parent 642)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 819)
  (parent 642)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 820)
  (parent 643)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) l-0)) (l-1 l-0)) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 821)
  (parent 643)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 822)
  (parent 643)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 3)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 823)
  (parent 643)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 824)
  (parent 645)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 825)
  (parent 647)
  (unrealized (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx) (w expt)
    (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 826)
  (parent 649)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 827)
  (parent 650)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l-0) (w (mul (rec x) l-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 828)
  (parent 650)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 829)
  (parent 650)
  (unrealized (0 3) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (6 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 830)
  (parent 650)
  (unrealized (0 3) (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-1) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-1) (pv self l))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 831)
  (parent 651)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1)))))
  (label 832)
  (parent 651)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 833)
  (parent 651)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 6 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1)))))
  (label 834)
  (parent 651)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 y x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-2))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-1 l-2)))))
  (label 835)
  (parent 651)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 836)
  (parent 652)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 837)
  (parent 652)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 838)
  (parent 652)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 839)
  (parent 652)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 840)
  (parent 653)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 841)
  (parent 653)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 842)
  (parent 653)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 843)
  (parent 653)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 844)
  (parent 654)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 845)
  (parent 655)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 846)
  (parent 655)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 847)
  (parent 655)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (7 0))
    ((8 2) (2 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 848)
  (parent 656)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (7 0))
    ((8 2) (2 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 849)
  (parent 656)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (7 0))
    ((8 2) (2 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 850)
  (parent 656)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (5 0))
    ((8 2) (3 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((8 1) (3 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 851)
  (parent 657)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (5 0))
    ((8 2) (3 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((8 1) (3 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 852)
  (parent 657)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (5 0))
    ((8 2) (3 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((8 1) (3 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 853)
  (parent 657)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-2))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2))))
  (label 854)
  (parent 658)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) l-2)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-2)) y))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-2)) y))
      (send (cat (exp (gen) (mul (rec x) l-2)) y))))
  (label 855)
  (parent 658)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y l-2)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-2)) x))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-2)) x))
      (send (cat (exp (gen) (mul y l-2)) x))))
  (label 856)
  (parent 658)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0)))))))
  (label 857)
  (parent 659)
  (unrealized (0 3) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 858)
  (parent 659)
  (unrealized (0 3) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) w))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) y w))))
    (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 859)
  (parent 659)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 860)
  (parent 659)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 861)
  (parent 659)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (2 2)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 862)
  (parent 659)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 2)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 863)
  (parent 659)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-2)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 864)
  (parent 659)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 865)
  (parent 660)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 866)
  (parent 660)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 867)
  (parent 660)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 868)
  (parent 660)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 869)
  (parent 661)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 870)
  (parent 661)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 871)
  (parent 661)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 872)
  (parent 661)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0)))))))
  (label 873)
  (parent 664)
  (unrealized (0 3) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 874)
  (parent 664)
  (unrealized (0 3) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 875)
  (parent 664)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 876)
  (parent 664)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (3 2))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 877)
  (parent 664)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (3 2)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 878)
  (parent 664)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 879)
  (parent 664)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp gy (mul x (rec y) (rec w))) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec y) (rec w))) w))
    (exp gy (mul x (rec y))) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp gy (mul x (rec y) (rec w))) w))
      (send (cat (exp gy (mul x (rec y) (rec w))) w))))
  (label 880)
  (parent 664)
  (unrealized (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 881)
  (parent 668)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 882)
  (parent 668)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 883)
  (parent 668)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 884)
  (parent 668)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 885)
  (parent 669)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 886)
  (parent 669)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 887)
  (parent 669)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 888)
  (parent 669)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l)))
  (label 889)
  (parent 670)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 890)
  (parent 671)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 891)
  (parent 671)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 892)
  (parent 671)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (2 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 893)
  (parent 672)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (2 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 894)
  (parent 672)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (2 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 895)
  (parent 672)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (7 0))
    ((8 2) (3 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((8 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 896)
  (parent 673)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (7 0))
    ((8 2) (3 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((8 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 897)
  (parent 673)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (7 0))
    ((8 2) (3 2)) ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((8 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 898)
  (parent 673)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-2))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2))))
  (label 899)
  (parent 674)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) l-2)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-2)) y))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-2)) y))
      (send (cat (exp (gen) (mul (rec x) l-2)) y))))
  (label 900)
  (parent 674)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y l-2)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-2)) x))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-2)) x))
      (send (cat (exp (gen) (mul y l-2)) x))))
  (label 901)
  (parent 674)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0)))))))
  (label 902)
  (parent 675)
  (unrealized (0 3) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 903)
  (parent 675)
  (unrealized (0 3) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) w))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) y w))))
    (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 904)
  (parent 675)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 905)
  (parent 675)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 906)
  (parent 675)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (2 2)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 907)
  (parent 675)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 2)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 908)
  (parent 675)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w l-2)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 909)
  (parent 675)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 910)
  (parent 676)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 911)
  (parent 676)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 912)
  (parent 676)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (gy ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 913)
  (parent 676)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 914)
  (parent 677)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 915)
  (parent 677)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 916)
  (parent 677)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gy gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 917)
  (parent 677)
  (unrealized (3 2) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0)))))))
  (label 918)
  (parent 680)
  (unrealized (0 3) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 919)
  (parent 680)
  (unrealized (0 3) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 920)
  (parent 680)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 921)
  (parent 680)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (2 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (3 2))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 922)
  (parent 680)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (3 2)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 923)
  (parent 680)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 924)
  (parent 680)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gy base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy gy) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp gy (mul x (rec y))))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp gy (mul x (rec y) (rec w))) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp gy (mul x (rec y) (rec w))) w))
    (exp gy (mul x (rec y))) (3 2))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat gy
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp gy (mul x (rec y)))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp gy x))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp gy (mul x (rec y) (rec w))) w))
      (send (cat (exp gy (mul x (rec y) (rec w))) w))))
  (label 925)
  (parent 680)
  (unrealized (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 926)
  (parent 696)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 927)
  (parent 696)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 928)
  (parent 696)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 929)
  (parent 697)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 930)
  (parent 698)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3))
    ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 931)
  (parent 700)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3))
    ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 932)
  (parent 700)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 933)
  (parent 700)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w))) w))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w))) w))
    (exp (gen) (rec x)) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) (mul (rec x) (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) (rec w))) w))))
  (label 934)
  (parent 703)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w))) w))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w))) w))
    (exp (gen) (rec x)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) (rec w))) w))))
  (label 935)
  (parent 704)
  (unrealized (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 936)
  (parent 705)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 8 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 937)
  (parent 705)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 938)
  (parent 705)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 1) (7 0))
    ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 939)
  (parent 705)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y)))))))
  (label 940)
  (parent 705)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 941)
  (parent 705)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 942)
  (parent 705)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))))
  (label 943)
  (parent 705)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 944)
  (parent 706)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 8 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 945)
  (parent 706)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 946)
  (parent 706)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-0))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 947)
  (parent 706)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y)))))))
  (label 948)
  (parent 706)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 949)
  (parent 706)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 950)
  (parent 706)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))))
  (label 951)
  (parent 706)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0))
    ((10 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b-1 (hash (exp ga l-2) (exp gx y)))))))
  (label 952)
  (parent 707)
  (unrealized (4 0) (7 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb-0)
    (priv-stor priv-stor-2) (l l-2) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0))
    ((10 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-1)))))
  (label 953)
  (parent 707)
  (unrealized (4 0) (7 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 954)
  (parent 707)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0)))))
  (label 955)
  (parent 707)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0)))))
  (label 956)
  (parent 707)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 2) (6 0)) ((8 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 957)
  (parent 707)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 10 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 958)
  (parent 707)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-2)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0))
    ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-2)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 959)
  (parent 707)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))))
  (label 960)
  (parent 707)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (3 0)) ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 961)
  (parent 708)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul x y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 962)
  (parent 709)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 963)
  (parent 709)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 964)
  (parent 709)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 965)
  (parent 710)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 966)
  (parent 710)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 967)
  (parent 710)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul x x)))
  (deflistener x)
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x))))
    ((recv x) (send x)))
  (label 968)
  (parent 711)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx) (w expt)
    (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 969)
  (parent 715)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 970)
  (parent 715)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w-0 (mul x w)) (w-1 w))
    (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 971)
  (parent 715)
  (unrealized (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul x w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (gen) (mul x w l-0))) (send (cat (gen) (mul x w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 972)
  (parent 715)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 973)
  (parent 715)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w l-0))) (send (cat (gen) (mul x w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 974)
  (parent 715)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w l-1))) (send (cat (gen) (mul x w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 975)
  (parent 715)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (8 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) (mul x (rec l) y)))
      (send (cat (exp (gen) l) (mul x (rec l) y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y)))))))
  (label 976)
  (parent 716)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (8 0)) ((7 2) (6 0))
    ((8 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 7 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y)))))))
  (label 977)
  (parent 716)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)) ((8 3) (6 0))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (6 0))
  (gen-st (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 978)
  (parent 716)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (8 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) (mul x (rec l) x-0)))
      (send (cat (exp (gen) l) (mul x (rec l) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 979)
  (parent 717)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x x-0 (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (8 0)) ((7 2) (6 0))
    ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 7 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x x-0 (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 980)
  (parent 717)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)) ((8 2) (6 0))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (6 0))
  (gen-st (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 981)
  (parent 717)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx) (w expt)
    (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-1) (pv a l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 982)
  (parent 722)
  (unrealized (3 0) (6 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) x x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) x x)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) x x))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 983)
  (parent 722)
  (unrealized (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec l-0) w)) (l-1 l-0) (w-1 w))
    (gen) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 984)
  (parent 722)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) (mul x (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul x (rec l) w l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 985)
  (parent 722)
  (unrealized (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-1) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y)))))))
  (label 986)
  (parent 722)
  (unrealized (3 0) (6 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((7 2) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x w)))
      (send (cat (exp (gen) l-0) (mul x w))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 987)
  (parent 722)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 988)
  (parent 722)
  (unrealized (3 0) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 8 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 989)
  (parent 723)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0)))))
  (label 990)
  (parent 723)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 991)
  (parent 723)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 3) (6 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 1) (7 0))
    ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 992)
  (parent 723)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 993)
  (parent 723)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y)))))))
  (label 994)
  (parent 723)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 995)
  (parent 723)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx)
    (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 996)
  (parent 723)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) x))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 8 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 997)
  (parent 724)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0)))))
  (label 998)
  (parent 724)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y)))))))
  (label 999)
  (parent 724)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) l-0))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1000)
  (parent 724)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 1001)
  (parent 724)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1002)
  (parent 724)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1003)
  (parent 724)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx)
    (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 1004)
  (parent 724)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx ga-0 gx-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-1) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0))
    ((10 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-1
            (hash (exp ga-0 l-2) (exp gx-0 y-0)))))))
  (label 1005)
  (parent 725)
  (unrealized (4 0) (7 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) x))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1006)
  (parent 725)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0))
    ((10 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-0)))))
  (label 1007)
  (parent 725)
  (unrealized (4 0) (7 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 1008)
  (parent 725)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 1009)
  (parent 725)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (3 0))
    ((7 3) (6 0)) ((8 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0))
  (gen-st (pv b-0 l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1010)
  (parent 725)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 10 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1011)
  (parent 725)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-2)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0))
    ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-2)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-2)))
      (send (cat (exp (gen) y) (mul x (rec y) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1012)
  (parent 725)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx)
    (w expt) (y l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 1013)
  (parent 725)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul x (rec l) y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul x (rec l) y)))
      (send (cat (exp (gen) l) (mul x (rec l) y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1014)
  (parent 726)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x y (rec l-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1015)
  (parent 726)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1016)
  (parent 726)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul x (rec l) x-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul x (rec l) x-0)))
      (send (cat (exp (gen) l) (mul x (rec l) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1017)
  (parent 727)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x x-0 (rec l-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x x-0 (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1018)
  (parent 727)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1019)
  (parent 727)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx) (w expt)
    (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1020)
  (parent 732)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w x x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) w x x)))
      (send (cat (exp (gen) l) (mul (rec l) w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1021)
  (parent 732)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec l-0) w)) (l-1 l-0) (w-1 w))
    (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1022)
  (parent 732)
  (unrealized (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul x (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul x (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1023)
  (parent 732)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y)))))))
  (label 1024)
  (parent 732)
  (unrealized (4 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x w)))
      (send (cat (exp (gen) l-0) (mul x w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1025)
  (parent 732)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1026)
  (parent 732)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y)))))))
  (label 1027)
  (parent 733)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y)))))))
  (label 1028)
  (parent 733)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (9 0)) ((8 2) (7 0))
    ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y)))))))
  (label 1029)
  (parent 733)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 3) (7 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-2)))))
  (label 1030)
  (parent 733)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1031)
  (parent 734)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1032)
  (parent 734)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x x-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (9 0)) ((8 2) (7 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x x-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x x-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1033)
  (parent 734)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 2) (7 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-2)))))
  (label 1034)
  (parent 734)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) (mul x (rec l-2) w y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (mul x (rec l-2) w y)))
      (send (cat (exp (gen) l-2) (mul x (rec l-2) w y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 1035)
  (parent 740)
  (unrealized (4 0) (7 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 rndx) (w expt) (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) (mul x (rec l-2) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (mul x (rec l-2) w x-0)))
      (send (cat (exp (gen) l-2) (mul x (rec l-2) w x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1036)
  (parent 740)
  (unrealized (4 0) (7 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec l-1) w)) (l-2 l-1) (w-1 w))
    (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1037)
  (parent 740)
  (unrealized (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w x x)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w x x)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w x x))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1038)
  (parent 740)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1039)
  (parent 740)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (6 0) (8 0))
  (gen-st (pv self-0 l-1))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1040)
  (parent 740)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x w)))
      (send (cat (exp (gen) l-1) (mul x w))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1041)
  (parent 740)
  (unrealized (4 0) (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 rndx) (w expt) (l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-2))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1042)
  (parent 740)
  (unrealized (4 0) (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 3) (4 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1043)
  (parent 741)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 3) (4 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1044)
  (parent 741)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 3) (4 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1045)
  (parent 742)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 3) (4 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1046)
  (parent 742)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x l l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (4 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1047)
  (parent 743)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x l l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (4 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1048)
  (parent 743)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (rec x)) x-0))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (4 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) x-0))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (rec x)) x-0))
      (send (cat (exp (gen) (rec x)) x-0))))
  (label 1049)
  (parent 744)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) x))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (4 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) x-0) x))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x))))
  (label 1050)
  (parent 744)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (rec x)) x-0))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (4 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) x-0))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (rec x)) x-0))
      (send (cat (exp (gen) (rec x)) x-0))))
  (label 1051)
  (parent 745)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) x))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (4 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) x-0) x))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x))))
  (label 1052)
  (parent 745)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) x-0))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (4 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) x-0))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) (rec x)) x-0))
      (send (cat (exp (gen) (rec x)) x-0))))
  (label 1053)
  (parent 746)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) x))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (4 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) x-0) x))
    (exp (gen) (mul (rec x) x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x))))
  (label 1054)
  (parent 746)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1055)
  (parent 747)
  (unrealized (0 3) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 1056)
  (parent 747)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 1057)
  (parent 747)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb self b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1058)
  (parent 747)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 1059)
  (parent 747)
  (unrealized (0 3) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))))
  (label 1060)
  (parent 747)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1061)
  (parent 747)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (3 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 5 ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor-0 (cat pt-3 (pv self l-0))))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0))))
  (label 1062)
  (parent 750)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0))
    ((8 2) (4 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-1 (cat pt-5 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 1063)
  (parent 752)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1064)
  (parent 754)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1065)
  (parent 754)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1066)
  (parent 754)
  (unrealized (0 3))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1067)
  (parent 755)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 1068)
  (parent 756)
  (unrealized (4 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1069)
  (parent 756)
  (unrealized (4 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul x x)))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x)))))
  (label 1070)
  (parent 756)
  (unrealized (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x l-0)))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x l-0))) (send (cat (gen) (mul x l-0)))))
  (label 1071)
  (parent 756)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul x l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul x l-0))) (send (cat (gen) (mul x l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1072)
  (parent 756)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x l-1))) (send (cat (gen) (mul x l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1073)
  (parent 756)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) w))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))))
  (label 1074)
  (parent 756)
  (unrealized (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (7 0))
    ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-1) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y)))))))
  (label 1075)
  (parent 757)
  (unrealized (3 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (7 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-1) (pv a l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1076)
  (parent 757)
  (unrealized (3 0) (7 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) x x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) x x)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) x x))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1077)
  (parent 757)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 8 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) x)) (send (cat (exp (gen) l-0) x)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1078)
  (parent 757)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul x (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (7 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul x (rec l) l-0)))
      (send (cat (exp (gen) l) (mul x (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1079)
  (parent 757)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (7 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1080)
  (parent 757)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w l-0)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))))
  (label 1081)
  (parent 757)
  (unrealized (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1082)
  (parent 758)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1083)
  (parent 758)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx)
    (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1084)
  (parent 758)
  (unrealized (0 3))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) y))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y)))))))
  (label 1085)
  (parent 759)
  (unrealized (4 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1086)
  (parent 759)
  (unrealized (4 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) x x)))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) x x)))
      (send (cat (exp (gen) l) (mul (rec l) x x)))))
  (label 1087)
  (parent 759)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) x))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) x)) (send (cat (exp (gen) l-0) x))))
  (label 1088)
  (parent 759)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul x (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul x (rec l) l-0)))
      (send (cat (exp (gen) l) (mul x (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1089)
  (parent 759)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1090)
  (parent 759)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w l-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w l-0 (rec w-0))) w-0))))
  (label 1091)
  (parent 759)
  (unrealized (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-2) (mul x (rec l-2) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-2) (mul x (rec l-2) y)))
      (send (cat (exp (gen) l-2) (mul x (rec l-2) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 1092)
  (parent 760)
  (unrealized (4 0) (8 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-2) (mul x (rec l-2) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-2) (mul x (rec l-2) x-0)))
      (send (cat (exp (gen) l-2) (mul x (rec l-2) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1093)
  (parent 760)
  (unrealized (4 0) (8 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) x x)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) x x)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) x x))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1094)
  (parent 760)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1095)
  (parent 760)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 2) (7 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (6 0) (7 0))
  (gen-st (pv self-0 l-1))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1096)
  (parent 760)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) x))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) x)) (send (cat (exp (gen) l-1) x)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1097)
  (parent 760)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-2)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-2)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1098)
  (parent 760)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w l-1 (rec w-0))) w-0))))
  (label 1099)
  (parent 760)
  (unrealized (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1100)
  (parent 797)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1101)
  (parent 799)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-1) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1102)
  (parent 801)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y l rndx) (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1103)
  (parent 802)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 1104)
  (parent 803)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 1105)
  (parent 803)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1106)
  (parent 803)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 1107)
  (parent 803)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 1108)
  (parent 804)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l-0) (w (mul (rec x) l-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 1109)
  (parent 804)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 1110)
  (parent 804)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 1111)
  (parent 804)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 1112)
  (parent 806)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 1113)
  (parent 808)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x w))) (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 1114)
  (parent 810)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0)))))
  (label 1115)
  (parent 811)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) l-0)) (l-1 l-0)) (one) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 1116)
  (parent 811)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0)))))
  (label 1117)
  (parent 811)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 1) (0 3)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1118)
  (parent 811)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor)
    (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 1119)
  (parent 812)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1))) (one) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1)))))
  (label 1120)
  (parent 812)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor-0) (l l-1)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-1) (x l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1)))))
  (label 1121)
  (parent 812)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1)))))
  (label 1122)
  (parent 812)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 y x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-2))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul l-1 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-2)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1123)
  (parent 812)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 1124)
  (parent 813)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 1125)
  (parent 813)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 1126)
  (parent 813)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 1127)
  (parent 813)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 1128)
  (parent 814)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 1129)
  (parent 814)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 1130)
  (parent 814)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 1131)
  (parent 814)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 1132)
  (parent 815)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 1133)
  (parent 815)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l-1) (x-0 l))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-0) (pv self l-1) (pv a l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 1134)
  (parent 815)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 1135)
  (parent 815)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-2))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 2) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)) ((9 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv a l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-0 l-2)))))
  (label 1136)
  (parent 815)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 1137)
  (parent 816)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 1138)
  (parent 817)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (5 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1))))
    ((recv x) (send x)))
  (label 1139)
  (parent 818)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 1140)
  (parent 820)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 1141)
  (parent 824)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 1142)
  (parent 824)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 1143)
  (parent 824)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((1 1) (6 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 1144)
  (parent 824)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 1145)
  (parent 825)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 1146)
  (parent 825)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (ga gx base) (priv-stor priv-stor-0 locn)
    (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 1147)
  (parent 825)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (7 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 3) (5 0)) ((7 1) (0 3)) ((8 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((4 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))))
  (label 1148)
  (parent 825)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 1149)
  (parent 826)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 1150)
  (parent 826)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l-1) (y-0 l))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-0) (pv self l-1) (pv b-0 l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 4 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 1151)
  (parent 826)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-1))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 1152)
  (parent 826)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-2))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-2))) (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 3) (5 0)) ((7 1) (6 0)) ((8 1) (0 3)) ((9 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (3 0) (5 0) (8 0))
  (gen-st (pv b l-1) (pv self l) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)) ((7 1) (6 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul y-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) l-2)) (send (cat (exp (gen) y-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-2))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self-0 l-2)))))
  (label 1153)
  (parent 826)
  (unrealized (0 3) (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (7 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (6 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (2 2)) ((6 1) (0 3)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 1154)
  (parent 829)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 2)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1 l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1))))
    ((recv x) (send x)))
  (label 1155)
  (parent 834)
  (unrealized (5 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1156)
  (parent 836)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1157)
  (parent 836)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1158)
  (parent 836)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1159)
  (parent 837)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1160)
  (parent 837)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1161)
  (parent 837)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1162)
  (parent 838)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1163)
  (parent 838)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1164)
  (parent 838)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1165)
  (parent 839)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1166)
  (parent 839)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1167)
  (parent 839)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1168)
  (parent 840)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1169)
  (parent 840)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1170)
  (parent 840)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1171)
  (parent 841)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1172)
  (parent 841)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1173)
  (parent 841)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1174)
  (parent 842)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1175)
  (parent 842)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1176)
  (parent 842)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1177)
  (parent 843)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1178)
  (parent 843)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1179)
  (parent 843)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 1180)
  (parent 844)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (7 0))
    ((8 2) (2 2)) ((9 1) (0 3)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1181)
  (parent 848)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (7 0))
    ((8 2) (2 2)) ((9 1) (0 3)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1182)
  (parent 848)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (5 0))
    ((8 2) (3 2)) ((9 1) (0 3)) ((10 1) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((8 1) (3 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1183)
  (parent 851)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (5 0))
    ((8 2) (3 2)) ((9 1) (0 3)) ((10 1) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((8 1) (3 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1184)
  (parent 851)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (11 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l) (pv self l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((9 1) (11 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-2 (10 0)
    (ch-msg priv-stor-2 (cat pt-8 (pv self l-2))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2)))
    ((load priv-stor-2 (cat pt-8 (pv self l-2)))
      (stor priv-stor-2 (cat pt-9 "nil")) (send l-2)))
  (label 1185)
  (parent 854)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1186)
  (parent 857)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1187)
  (parent 857)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1188)
  (parent 857)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1189)
  (parent 857)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1190)
  (parent 858)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1191)
  (parent 858)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1192)
  (parent 858)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1193)
  (parent 858)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) x))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 1194)
  (parent 859)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) w))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 1195)
  (parent 859)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (8 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1196)
  (parent 860)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0))))))
  (label 1197)
  (parent 861)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1198)
  (parent 861)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-0) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (2 2)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (8 0))
  (gen-st (pv b l-1) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1199)
  (parent 862)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (2 2)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1200)
  (parent 862)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 2)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1201)
  (parent 863)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 2)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1202)
  (parent 863)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l l-0)) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) l-2))
      (send (cat (exp (gen) (mul l l-0)) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x (rec l-1)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1203)
  (parent 864)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-2)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))))
  (label 1204)
  (parent 864)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1205)
  (parent 873)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1206)
  (parent 873)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1207)
  (parent 873)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1208)
  (parent 873)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1209)
  (parent 874)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1210)
  (parent 874)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1211)
  (parent 874)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1212)
  (parent 874)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (8 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1213)
  (parent 875)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 1214)
  (parent 876)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 1215)
  (parent 876)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 1216)
  (parent 876)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (2 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 1217)
  (parent 877)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (2 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 1218)
  (parent 877)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (2 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 1219)
  (parent 877)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 1220)
  (parent 878)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 1221)
  (parent 878)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 1222)
  (parent 878)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-2))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2))))
  (label 1223)
  (parent 879)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) l-2)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-2)) y))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-2)) y))
      (send (cat (exp (gen) (mul (rec x) l-2)) y))))
  (label 1224)
  (parent 879)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y l-2)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-2)) x))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-2)) x))
      (send (cat (exp (gen) (mul y l-2)) x))))
  (label 1225)
  (parent 879)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0)))))))
  (label 1226)
  (parent 880)
  (unrealized (0 3) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1227)
  (parent 880)
  (unrealized (0 3) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) w))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) y w))))
    (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1228)
  (parent 880)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1229)
  (parent 880)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1230)
  (parent 880)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (2 2))
    ((10 1) (0 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0) (9 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((10 1) (0 0)) ((10 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1231)
  (parent 880)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 2))
    ((10 1) (3 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((10 1) (3 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1232)
  (parent 880)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-2)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1233)
  (parent 880)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1234)
  (parent 881)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1235)
  (parent 881)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1236)
  (parent 881)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1237)
  (parent 882)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1238)
  (parent 882)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1239)
  (parent 882)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1240)
  (parent 883)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1241)
  (parent 883)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 3) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1242)
  (parent 883)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1243)
  (parent 884)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1244)
  (parent 884)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 3) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1245)
  (parent 884)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1246)
  (parent 885)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1247)
  (parent 885)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1248)
  (parent 885)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1249)
  (parent 886)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1250)
  (parent 886)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1251)
  (parent 886)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1252)
  (parent 887)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1253)
  (parent 887)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1254)
  (parent 887)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1255)
  (parent 888)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1256)
  (parent 888)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 2) (3 2)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1257)
  (parent 888)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l)))
  (label 1258)
  (parent 889)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (2 2)) ((9 1) (0 3)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1259)
  (parent 893)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (2 2)) ((9 1) (0 3)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1260)
  (parent 893)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (7 0))
    ((8 2) (3 2)) ((9 1) (0 3)) ((10 1) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((8 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1261)
  (parent 896)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 0)) ((8 1) (7 0))
    ((8 2) (3 2)) ((9 1) (0 3)) ((10 1) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((8 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1262)
  (parent 896)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (11 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((9 1) (11 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-2 (10 0)
    (ch-msg priv-stor-2 (cat pt-8 (pv self l-2))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2)))
    ((load priv-stor-2 (cat pt-8 (pv self l-2)))
      (stor priv-stor-2 (cat pt-9 "nil")) (send l-2)))
  (label 1263)
  (parent 899)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1264)
  (parent 902)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1265)
  (parent 902)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1266)
  (parent 902)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1267)
  (parent 902)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1268)
  (parent 903)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1269)
  (parent 903)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1270)
  (parent 903)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1271)
  (parent 903)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) x))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 1272)
  (parent 904)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) w))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 1273)
  (parent 904)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (8 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1274)
  (parent 905)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0))))))
  (label 1275)
  (parent 906)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1276)
  (parent 906)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-0) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (2 2)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (8 0))
  (gen-st (pv b l-1) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1277)
  (parent 907)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (2 2)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1278)
  (parent 907)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 2)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1279)
  (parent 908)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 2)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1280)
  (parent 908)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x (rec l-1)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1281)
  (parent 909)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w l-2)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))))
  (label 1282)
  (parent 909)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1283)
  (parent 918)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1284)
  (parent 918)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1285)
  (parent 918)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1286)
  (parent 918)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1287)
  (parent 919)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1288)
  (parent 919)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1289)
  (parent 919)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1290)
  (parent 919)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (8 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1291)
  (parent 920)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 1292)
  (parent 921)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 1293)
  (parent 921)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 1294)
  (parent 921)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (2 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 1295)
  (parent 922)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (2 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 1296)
  (parent 922)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (2 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 1297)
  (parent 922)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-1))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1))))
  (label 1298)
  (parent 923)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) l-1)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-1)) y))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-1)) y))
      (send (cat (exp (gen) (mul (rec x) l-1)) y))))
  (label 1299)
  (parent 923)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul y l-1)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-1)) x))
    (exp (gen) (mul (rec x) y l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-1)) x))
      (send (cat (exp (gen) (mul y l-1)) x))))
  (label 1300)
  (parent 923)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) l-2))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2))))
  (label 1301)
  (parent 924)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) l-2)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) l-2)) y))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) l-2)) y))
      (send (cat (exp (gen) (mul (rec x) l-2)) y))))
  (label 1302)
  (parent 924)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y l-2)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y l-2)) x))
    (exp (gen) (mul (rec x) y l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul y l-2)) x))
      (send (cat (exp (gen) (mul y l-2)) x))))
  (label 1303)
  (parent 924)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0)))))))
  (label 1304)
  (parent 925)
  (unrealized (0 3) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1305)
  (parent 925)
  (unrealized (0 3) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) w))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (gy (exp (gen) (mul (rec x) y w))))
    (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1306)
  (parent 925)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1307)
  (parent 925)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1308)
  (parent 925)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (2 2))
    ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0) (9 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1309)
  (parent 925)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 2))
    ((10 1) (3 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((10 1) (3 0)) ((10 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1310)
  (parent 925)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w l-2)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1311)
  (parent 925)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (8 0))
  (gen-st (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1312)
  (parent 934)
  (unrealized (7 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 1313)
  (parent 934)
  (unrealized (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (gen) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 1314)
  (parent 934)
  (unrealized (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l (rec x)))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec x)) l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l))
      (send (cat (exp (gen) (rec x)) l)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))))
  (label 1315)
  (parent 934)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (8 0))
  (gen-st (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 1316)
  (parent 934)
  (unrealized (7 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((6 2) (8 0)) ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 6 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))))
  (label 1317)
  (parent 934)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1318)
  (parent 934)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y)))))))
  (label 1319)
  (parent 935)
  (unrealized (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-2))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1320)
  (parent 935)
  (unrealized (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 1321)
  (parent 935)
  (unrealized (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (10 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 1322)
  (parent 935)
  (unrealized (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))))
  (label 1323)
  (parent 935)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (8 0)) ((6 2) (3 0)) ((7 1) (0 3)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (9 0))
  (gen-st (pv self l-0) (pv self-0 l-1))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((6 1) (8 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1324)
  (parent 935)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((7 2) (10 0)) ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 7 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))))
  (label 1325)
  (parent 935)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1326)
  (parent 935)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb-0)
    (priv-stor priv-stor) (l l-0) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-1)))))
  (label 1327)
  (parent 936)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb-0)
    (priv-stor priv-stor-0) (l l-0) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l) (x-1 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-1)))))
  (label 1328)
  (parent 936)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 2) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1329)
  (parent 936)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1330)
  (parent 940)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1331)
  (parent 940)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 3) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-1 l-1)))))
  (label 1332)
  (parent 940)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 1333)
  (parent 943)
  (unrealized (4 0) (7 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 init 3) (exp (gen) x-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 1334)
  (parent 943)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) w)) (x-2 x-0) (w-1 w))
    (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1335)
  (parent 943)
  (unrealized (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul w x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 1336)
  (parent 943)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1337)
  (parent 943)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 3) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y)))))))
  (label 1338)
  (parent 943)
  (unrealized (4 0) (7 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1339)
  (parent 943)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1340)
  (parent 943)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb-0)
    (priv-stor priv-stor) (l l-0) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l) (x-1 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-1)))))
  (label 1341)
  (parent 944)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb-0)
    (priv-stor priv-stor-0) (l l-0) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-1)))))
  (label 1342)
  (parent 944)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 2) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1343)
  (parent 944)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1344)
  (parent 948)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1345)
  (parent 948)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 3) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-1 l-1)))))
  (label 1346)
  (parent 948)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 1347)
  (parent 951)
  (unrealized (4 0) (7 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 init 3) (exp (gen) x-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 1348)
  (parent 951)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) w)) (x-2 x-0) (w-1 w))
    (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1349)
  (parent 951)
  (unrealized (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul w x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 1350)
  (parent 951)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-0))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1351)
  (parent 951)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 3) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y)))))))
  (label 1352)
  (parent 951)
  (unrealized (4 0) (7 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1353)
  (parent 951)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1354)
  (parent 951)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (8 0)) ((10 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b (hash (exp ga l-1) (exp gx y)))))))
  (label 1355)
  (parent 952)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (8 0)) ((10 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l-1) (x-0 l-0) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 a (hash (exp ga l-1) (exp gx y)))))))
  (label 1356)
  (parent 952)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b a-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0))
    ((9 1) (10 0)) ((10 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((9 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 a-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 1357)
  (parent 952)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 na-1 nb-0 data) (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0))
    ((10 3) (7 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b-1 (hash (exp ga l-2) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-1 l-2)))))
  (label 1358)
  (parent 952)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb-0)
    (priv-stor priv-stor-0) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (8 0)) ((10 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-1)))))
  (label 1359)
  (parent 953)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb-0)
    (priv-stor priv-stor) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (8 0)) ((10 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l-1) (x-0 l-0) (x-1 l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-1)))))
  (label 1360)
  (parent 953)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0))
    ((9 1) (10 0)) ((10 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((9 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 1361)
  (parent 953)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 x-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb-0)
    (priv-stor priv-stor-2) (l l-2) (x x-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0)) ((9 1) (8 0))
    ((10 2) (7 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-1))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-1 l-2)))))
  (label 1362)
  (parent 953)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 rndx) (w expt) (x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb-0)
    (priv-stor priv-stor-2) (l l-2) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (7 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-1)))))
  (label 1363)
  (parent 960)
  (unrealized (4 0) (7 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 init 3) (exp (gen) x-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 1364)
  (parent 960)
  (unrealized (4 0) (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) w)) (x-2 x-0) (w-1 w))
    (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1365)
  (parent 960)
  (unrealized (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul w x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 1366)
  (parent 960)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1367)
  (parent 960)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 3) (7 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b-1 (hash (exp ga l-2) (exp gx y)))))))
  (label 1368)
  (parent 960)
  (unrealized (4 0) (7 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1369)
  (parent 960)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1370)
  (parent 960)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-2))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-2)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-2))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1371)
  (parent 960)
  (unrealized (4 0) (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1372)
  (parent 961)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1373)
  (parent 969)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1374)
  (parent 969)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1375)
  (parent 969)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))))
  (label 1376)
  (parent 970)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (w (mul (rec x) (rec x))) (x-0 x))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 1377)
  (parent 970)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))))
  (label 1378)
  (parent 970)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1379)
  (parent 970)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) w))
  (deflistener x)
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul w x x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w)))
    ((recv x) (send x)))
  (label 1380)
  (parent 970)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul (rec x) l-0)))
      (send (cat (gen) (mul (rec x) l-0)))))
  (label 1381)
  (parent 971)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 1382)
  (parent 971)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul (rec x) l-0)))
      (send (cat (gen) (mul (rec x) l-0)))))
  (label 1383)
  (parent 971)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul (rec x) l-1)))
      (send (cat (gen) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1384)
  (parent 971)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) w))
  (deflistener x)
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))) ((recv x) (send x)))
  (label 1385)
  (parent 971)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (7 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1386)
  (parent 972)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec l-0))) (l-1 l-0)) (one)
    (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb self b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1387)
  (parent 972)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 8 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1388)
  (parent 972)
  (unrealized (0 3) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 1389)
  (parent 972)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1390)
  (parent 973)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1391)
  (parent 973)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx)
    (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1392)
  (parent 973)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))))
  (label 1393)
  (parent 974)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec l-0))) (l-1 l-0)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))))
  (label 1394)
  (parent 974)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x)))))
  (label 1395)
  (parent 974)
  (unrealized (0 3) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1396)
  (parent 974)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1397)
  (parent 975)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec l-1))) (l-2 l-1)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1398)
  (parent 975)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1399)
  (parent 975)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (rec x)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (rec x)))
      (send (cat (exp (gen) l-1) (rec x))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1400)
  (parent 975)
  (unrealized (0 3) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1401)
  (parent 975)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) (mul x (rec l) w x-0)))
      (send (cat (exp (gen) l) (mul x (rec l) w x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1402)
  (parent 982)
  (unrealized (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x w x-0 (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 7 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x w x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w x-0 (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1403)
  (parent 982)
  (unrealized (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx)
    (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 1404)
  (parent 982)
  (unrealized (3 0) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l (rec x) (rec x) l-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul l (rec x) (rec x) l-0)))))
  (label 1405)
  (parent 983)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x) (rec x))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (w (mul l-0 (rec x) (rec x))) (l-1 l-0) (x-0 x)) (one)
    (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x) (rec x))))))
  (label 1406)
  (parent 983)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-0 l-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-0 l-0)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-0 l-0)))))
  (label 1407)
  (parent 983)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x) l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 1408)
  (parent 983)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul (rec x) l l-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) l l-0)))
      (send (cat (gen) (mul (rec x) l l-0)))))
  (label 1409)
  (parent 984)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) l-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l-0) (w (mul (rec x) l-0))) (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) l-0)))
      (send (cat (gen) (mul (rec x) l-0)))))
  (label 1410)
  (parent 984)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) l-0 l-0)))
      (send (cat (gen) (mul (rec x) l-0 l-0)))))
  (label 1411)
  (parent 984)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) l-0 l-1)))
      (send (cat (gen) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 1412)
  (parent 984)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l)))
      (send (cat (exp (gen) l-0) (mul (rec x) l)))))
  (label 1413)
  (parent 985)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l (rec l-0))))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l) (w (mul (rec x) l (rec l-0))) (l-2 l-0))
    (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) l (rec l-0))))))
  (label 1414)
  (parent 985)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) (rec l) l-0 l-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul (rec x) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul (rec x) (rec l) l-0 l-0)))))
  (label 1415)
  (parent 985)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 1416)
  (parent 985)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) (mul x (rec l) w y)))
      (send (cat (exp (gen) l) (mul x (rec l) w y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y)))))))
  (label 1417)
  (parent 986)
  (unrealized (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x w y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 7 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x w y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y)))))))
  (label 1418)
  (parent 986)
  (unrealized (3 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx)
    (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1419)
  (parent 986)
  (unrealized (3 0) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec x) l-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((7 2) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul (rec x) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) l-0)))))
  (label 1420)
  (parent 987)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((7 2) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x)))))
  (label 1421)
  (parent 987)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l-0)))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((7 2) (8 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 7 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l-0)))
      (send (cat (exp (gen) l-0) (mul (rec x) l-0)))))
  (label 1422)
  (parent 987)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x) l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((7 2) (8 0)) ((8 1) (0 3)) ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 1423)
  (parent 987)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener x)
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((7 2) (8 0)) ((8 1) (0 3)) ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x w)))
      (send (cat (exp (gen) l-0) (mul x w))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv x) (send x)))
  (label 1424)
  (parent 987)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l l-1))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1425)
  (parent 988)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-0) (w (mul (rec x) l-0 (rec l-1)))
      (l-3 l-1)) (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1426)
  (parent 988)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (6 0) (8 0))
  (gen-st (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1427)
  (parent 988)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1428)
  (parent 988)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l x l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-2))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-1 l-2)))))
  (label 1429)
  (parent 988)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx gb base)
    (priv-stor priv-stor-0 locn) (l x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand init 3 (na na-1) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1430)
  (parent 990)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx gb base)
    (priv-stor priv-stor-0 locn) (l x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (defstrand init 3 (na na-1) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1431)
  (parent 990)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 2) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-1 l-1)))))
  (label 1432)
  (parent 990)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 locn)
    (l x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b a) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 a (hash (exp ga-0 l-0) (exp gx-0 y-0)))))))
  (label 1433)
  (parent 993)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 locn)
    (l x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b (hash (exp ga-0 l-0) (exp gx-0 y-0)))))))
  (label 1434)
  (parent 993)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 3) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1435)
  (parent 993)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1436)
  (parent 996)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y rndx) (w expt) (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l-0) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0)))))
  (label 1437)
  (parent 996)
  (unrealized (4 0) (7 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 x y rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec y) w)) (y-0 y) (w-1 w)) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1438)
  (parent 996)
  (unrealized (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y rndx) (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1439)
  (parent 996)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1440)
  (parent 996)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y rndx) (w expt) (y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 3) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l-0) (y-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 1441)
  (parent 996)
  (unrealized (4 0) (7 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1442)
  (parent 996)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1443)
  (parent 996)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx gb base)
    (priv-stor priv-stor-0 locn) (l x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand init 3 (na na-1) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1444)
  (parent 998)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx gb base)
    (priv-stor priv-stor-0 locn) (l x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand init 3 (na na-1) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1445)
  (parent 998)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 2) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-1 l-1)))))
  (label 1446)
  (parent 998)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 locn)
    (l x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b a) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 a (hash (exp ga-0 l-0) (exp gx-0 y-0)))))))
  (label 1447)
  (parent 1001)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 locn)
    (l x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b (hash (exp ga-0 l-0) (exp gx-0 y-0)))))))
  (label 1448)
  (parent 1001)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 3) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1449)
  (parent 1001)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1450)
  (parent 1004)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y rndx) (w expt) (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0)))))
  (label 1451)
  (parent 1004)
  (unrealized (4 0) (7 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 x y rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec y) w)) (y-0 y) (w-1 w)) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1452)
  (parent 1004)
  (unrealized (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y rndx) (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1453)
  (parent 1004)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-0))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1454)
  (parent 1004)
  (unrealized (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y rndx) (w expt) (y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 3) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 1455)
  (parent 1004)
  (unrealized (4 0) (7 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1456)
  (parent 1004)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1457)
  (parent 1004)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 1) (8 0)) ((10 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 1458)
  (parent 1005)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b a) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 1) (8 0)) ((10 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 a (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 1459)
  (parent 1005)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0))
    ((9 1) (10 0)) ((10 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((9 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 1460)
  (parent 1005)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx ga-0 gx-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-1) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0))
    ((10 3) (7 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-1
            (hash (exp ga-0 l-2) (exp gx-0 y-0))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-1 l-2)))))
  (label 1461)
  (parent 1005)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 1) (8 0)) ((10 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-0)))))
  (label 1462)
  (parent 1007)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((9 1) (8 0)) ((10 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-0)))))
  (label 1463)
  (parent 1007)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a b-0) (b b-1) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0))
    ((9 1) (10 0)) ((10 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((9 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b-0 b-1 (exp (gen) x-0)))))
  (label 1464)
  (parent 1007)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0)) ((9 1) (8 0))
    ((10 2) (7 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-2))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-1 l-2)))))
  (label 1465)
  (parent 1007)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx)
    (w expt) (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1466)
  (parent 1013)
  (unrealized (4 0) (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x y rndx) (w expt) (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (7 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-0)))))
  (label 1467)
  (parent 1013)
  (unrealized (4 0) (7 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec y) w)) (y-0 y) (w-1 w)) (gen)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1468)
  (parent 1013)
  (unrealized (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y rndx)
    (w expt) (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1469)
  (parent 1013)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b-0 l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1470)
  (parent 1013)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx ga-0 gx-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x y rndx) (w expt) (y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-1) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 3) (7 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-1
            (hash (exp ga-0 l-2) (exp gx-0 y-0)))))))
  (label 1471)
  (parent 1013)
  (unrealized (4 0) (7 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1472)
  (parent 1013)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1473)
  (parent 1013)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y rndx) (w expt) (l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-2))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w l-2)))
      (send (cat (exp (gen) y) (mul x (rec y) w l-2))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1474)
  (parent 1013)
  (unrealized (4 0) (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul x (rec l) w x-0)))
      (send (cat (exp (gen) l) (mul x (rec l) w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1475)
  (parent 1020)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x w x-0 (rec l-0))))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x w x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w x-0 (rec l-0)))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1476)
  (parent 1020)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1477)
  (parent 1020)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x) (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) x) (mul l (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul l (rec x) (rec x) l-0)))))
  (label 1478)
  (parent 1021)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x) (rec x))))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (l-1 l-0) (w (mul l-0 (rec x) (rec x))) (x-0 x)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x) (rec x))))))
  (label 1479)
  (parent 1021)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-0 l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-0 l-0)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-0 l-0)))))
  (label 1480)
  (parent 1021)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-0 (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1481)
  (parent 1021)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul (rec x) l l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul (rec x) l l-0)))
      (send (cat (gen) (mul (rec x) l l-0)))))
  (label 1482)
  (parent 1022)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l-0) (w (mul (rec x) l-0))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul (rec x) l-0)))
      (send (cat (gen) (mul (rec x) l-0)))))
  (label 1483)
  (parent 1022)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul (rec x) l-0 l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul (rec x) l-0 l-0)))
      (send (cat (gen) (mul (rec x) l-0 l-0)))))
  (label 1484)
  (parent 1022)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul (rec x) l-0 l-1)))
      (send (cat (gen) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1485)
  (parent 1022)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) (rec l) l-0 l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (7 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec x) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul (rec x) (rec l) l-0 l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1486)
  (parent 1023)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l) (w (mul (rec x) l (rec l-0))) (l-2 l-0))
    (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1487)
  (parent 1023)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 8 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l)))
      (send (cat (exp (gen) l-0) (mul (rec x) l))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1488)
  (parent 1023)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 1489)
  (parent 1023)
  (unrealized (0 3) (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul x (rec l) w y)))
      (send (cat (exp (gen) l) (mul x (rec l) w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1490)
  (parent 1024)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x w y (rec l-0))))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x w y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w y (rec l-0)))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1491)
  (parent 1024)
  (unrealized (4 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx)
    (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1492)
  (parent 1024)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec x) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) l-0)))))
  (label 1493)
  (parent 1025)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x)))))
  (label 1494)
  (parent 1025)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l-0)))
      (send (cat (exp (gen) l-0) (mul (rec x) l-0)))))
  (label 1495)
  (parent 1025)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-0 (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1496)
  (parent 1025)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x w)))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener x)
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x w)))
      (send (cat (exp (gen) l-0) (mul x w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv x) (send x)))
  (label 1497)
  (parent 1025)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1498)
  (parent 1026)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-0) (w (mul (rec x) l-0 (rec l-1)))
      (l-3 l-1)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1499)
  (parent 1026)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1500)
  (parent 1026)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1501)
  (parent 1026)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l x l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1502)
  (parent 1026)
  (unrealized (0 3) (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx)
    (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y)))))))
  (label 1503)
  (parent 1035)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx)
    (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y)))))))
  (label 1504)
  (parent 1035)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx)
    (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x w y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x w y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x w y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y)))))))
  (label 1505)
  (parent 1035)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 rndx) (w expt) (y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-2)))))
  (label 1506)
  (parent 1035)
  (unrealized (4 0) (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx)
    (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1507)
  (parent 1036)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx)
    (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1508)
  (parent 1036)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx)
    (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x w x-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x w x-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x w x-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1509)
  (parent 1036)
  (unrealized (4 0) (7 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 rndx) (w expt) (x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-2))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-2)))))
  (label 1510)
  (parent 1036)
  (unrealized (4 0) (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) l-0 l-1)))
      (send (cat (gen) (mul (rec x) l-0 l-1)))))
  (label 1511)
  (parent 1037)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) l-1)))
      (send (cat (gen) (mul (rec x) l-1)))))
  (label 1512)
  (parent 1037)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) l-0 l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) l-0 l-1)))
      (send (cat (gen) (mul (rec x) l-0 l-1)))))
  (label 1513)
  (parent 1037)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) l-1 l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) l-1 l-1)))
      (send (cat (gen) (mul (rec x) l-1 l-1)))))
  (label 1514)
  (parent 1037)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) l-1 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) l-1 l-2)))
      (send (cat (gen) (mul (rec x) l-1 l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1515)
  (parent 1037)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x) l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))))
  (label 1516)
  (parent 1038)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-1 (rec x) (rec x))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (l-2 l-1) (w (mul l-1 (rec x) (rec x))) (x-0 x)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l-1 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul l-1 (rec x) (rec x))))))
  (label 1517)
  (parent 1038)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul l-0 (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))))
  (label 1518)
  (parent 1038)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-1 l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-1 l-1)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-1 l-1)))))
  (label 1519)
  (parent 1038)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-1 (rec x) l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-1 (rec x) (rec x) l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l-1 (rec x) (rec x) l-2)))
      (send (cat (exp (gen) x) (mul l-1 (rec x) (rec x) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1520)
  (parent 1038)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))))
  (label 1521)
  (parent 1039)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-0) (w (mul (rec x) l-0 (rec l-1)))
      (l-3 l-1)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1))))))
  (label 1522)
  (parent 1039)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0)))))
  (label 1523)
  (parent 1039)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1)))))
  (label 1524)
  (parent 1039)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l x l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1525)
  (parent 1039)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (gb (exp (gen) l-1)) (gy (exp (gen) (mul (rec x) l l-1)))
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self-0 l-0))
  (facts (neq self-0 b) (undisclosed l-1))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-1))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l l-1)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1526)
  (parent 1040)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (6 0) (8 0))
  (gen-st (pv self-0 l-1))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-0) (w (mul (rec x) l-0 (rec l-1)))
      (l-3 l-1)) (one) (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1527)
  (parent 1040)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor-1)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self-0 l-0))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1528)
  (parent 1040)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (6 0) (8 0))
  (gen-st (pv self-0 l-1))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1529)
  (parent 1040)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l x l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-2))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)) ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l-1))
  (precur (6 0) (8 0))
  (gen-st (pv self-0 l-1))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-2))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-1 l-2)))))
  (label 1530)
  (parent 1040)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul l-0 (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l-1)))))
  (label 1531)
  (parent 1041)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x) l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) l-1)))))
  (label 1532)
  (parent 1041)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (rec x)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (rec x)))
      (send (cat (exp (gen) l-1) (rec x)))))
  (label 1533)
  (parent 1041)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-1)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-1)))))
  (label 1534)
  (parent 1041)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x) l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1535)
  (parent 1041)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener x)
  (precedes ((0 2) (9 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x w)))
      (send (cat (exp (gen) l-1) (mul x w))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv x) (send x)))
  (label 1536)
  (parent 1041)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l x l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-2))
    (gy (exp (gen) (mul (rec x) l-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-2))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-2))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1537)
  (parent 1042)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l x l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-2))) (priv-stor priv-stor) (l l-2)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-2))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l-2))
  (precur (7 0) (9 0))
  (gen-st (pv a l-2))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-2))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 l-2)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor (cat pt (pv a l-2)))
      (stor priv-stor (cat pt-3 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) l-0 (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1538)
  (parent 1042)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-3 l-1) (w (mul (rec x) l-1 (rec l-2)))
      (l-4 l-2)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1539)
  (parent 1042)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1) l-2 l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1540)
  (parent 1042)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) l-1)))
      (send (cat (exp (gen) l-2) (mul (rec x) l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1541)
  (parent 1042)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg) (na nb data)
    (a b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (l l-0 x l-1 l-2 l-3 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-3))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-3))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-3))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-3) (self self-1)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2 l-3)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-3))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-3)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-3))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-3)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-3))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-3)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec x) l-1 (rec l-2) l-3))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv self-1 l-3)))))
  (label 1542)
  (parent 1042)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 3)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1543)
  (parent 1055)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 3)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1544)
  (parent 1055)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3))
    ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1545)
  (parent 1055)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 1546)
  (parent 1056)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 1547)
  (parent 1057)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 3)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1548)
  (parent 1059)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 3)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1549)
  (parent 1059)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3))
    ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1550)
  (parent 1059)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w))) w))
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (3 0)) ((8 1) (0 3)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w))) w))
    (exp (gen) (rec x)) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) (mul (rec x) (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) (rec w))) w))))
  (label 1551)
  (parent 1062)
  (unrealized (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w))) w))
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0))
    ((8 2) (4 0)) ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w))) w))
    (exp (gen) (rec x)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) (rec w))) w))))
  (label 1552)
  (parent 1063)
  (unrealized (11 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 1553)
  (parent 1064)
  (unrealized (4 0) (8 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1554)
  (parent 1064)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 1555)
  (parent 1064)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 2) (7 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 1) (8 0)) ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1556)
  (parent 1064)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y)))))))
  (label 1557)
  (parent 1064)
  (unrealized (4 0) (8 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1558)
  (parent 1064)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1559)
  (parent 1064)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))))
  (label 1560)
  (parent 1064)
  (unrealized (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 1561)
  (parent 1065)
  (unrealized (4 0) (8 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1562)
  (parent 1065)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 1563)
  (parent 1065)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (8 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-0))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1564)
  (parent 1065)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y)))))))
  (label 1565)
  (parent 1065)
  (unrealized (4 0) (8 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1566)
  (parent 1065)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1567)
  (parent 1065)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))))
  (label 1568)
  (parent 1065)
  (unrealized (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)) ((11 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-2))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b-1 (hash (exp ga l-2) (exp gx y)))))))
  (label 1569)
  (parent 1066)
  (unrealized (4 0) (8 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb-0)
    (priv-stor priv-stor-2) (l l-2) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)) ((11 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-2))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-1)))))
  (label 1570)
  (parent 1066)
  (unrealized (4 0) (8 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1571)
  (parent 1066)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0)))))
  (label 1572)
  (parent 1066)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0)))))
  (label 1573)
  (parent 1066)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 2) (7 0)) ((9 1) (8 0)) ((10 1) (0 0))
    ((10 1) (5 0)) ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (8 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1574)
  (parent 1066)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)) ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 11 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1575)
  (parent 1066)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-2)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)) ((11 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-2)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1576)
  (parent 1066)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (11 0)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0) (11 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))))
  (label 1577)
  (parent 1066)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (5 0) (6 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1578)
  (parent 1067)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul x y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1579)
  (parent 1068)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1580)
  (parent 1068)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1581)
  (parent 1068)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1582)
  (parent 1069)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1583)
  (parent 1069)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1584)
  (parent 1069)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul x x)))
  (deflistener x)
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x x) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x))))
    ((recv x) (send x)))
  (label 1585)
  (parent 1070)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx) (w expt)
    (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 2) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1586)
  (parent 1074)
  (unrealized (4 0) (8 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1587)
  (parent 1074)
  (unrealized (4 0) (8 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w-0 (mul x w)) (w-1 w))
    (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1588)
  (parent 1074)
  (unrealized (8 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul x w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul x w l-0))) (send (cat (gen) (mul x w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1589)
  (parent 1074)
  (unrealized (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 3) (8 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 1590)
  (parent 1074)
  (unrealized (4 0) (8 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w l-0))) (send (cat (gen) (mul x w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1591)
  (parent 1074)
  (unrealized (4 0) (8 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w l-1))) (send (cat (gen) (mul x w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1592)
  (parent 1074)
  (unrealized (4 0) (8 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul x (rec l) y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 1) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (6 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (9 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul x (rec l) y)))
      (send (cat (exp (gen) l) (mul x (rec l) y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y)))))))
  (label 1593)
  (parent 1075)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 1) (9 0))
    ((8 2) (7 0)) ((9 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)) ((8 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 8 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y)))))))
  (label 1594)
  (parent 1075)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (7 0))
    ((9 3) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1595)
  (parent 1075)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul x (rec l) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 1) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (6 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (9 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul x (rec l) x-0)))
      (send (cat (exp (gen) l) (mul x (rec l) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1596)
  (parent 1076)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x x-0 (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 1) (9 0))
    ((8 2) (7 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)) ((8 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 8 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x x-0 (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1597)
  (parent 1076)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0)) ((8 2) (7 0))
    ((9 2) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 1598)
  (parent 1076)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx) (w expt)
    (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-1) (pv a l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1599)
  (parent 1081)
  (unrealized (3 0) (7 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) x x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) x x)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) x x))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1600)
  (parent 1081)
  (unrealized (3 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(comment "Step limit exceeded--aborting run")

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 rndx) (w expt) (l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-2))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1659)
  (parent 1099)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x w)))
      (send (cat (exp (gen) l-1) (mul x w))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1658)
  (parent 1099)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self-0 l-1))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1657)
  (parent 1099)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 1656)
  (parent 1099)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w x x)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w x x)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w x x))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1655)
  (parent 1099)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec l-1) w)) (l-2 l-1) (w-1 w))
    (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1654)
  (parent 1099)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 rndx) (w expt) (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-2) (mul x (rec l-2) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-2) (mul x (rec l-2) w x-0)))
      (send (cat (exp (gen) l-2) (mul x (rec l-2) w x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1653)
  (parent 1099)
  (unrealized (4 0) (8 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-2) (mul x (rec l-2) w y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (8 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-2) (mul x (rec l-2) w y)))
      (send (cat (exp (gen) l-2) (mul x (rec l-2) w y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 1652)
  (parent 1099)
  (unrealized (4 0) (8 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 2) (8 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-2))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-2)))))
  (label 1651)
  (parent 1093)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x x-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 1) (10 0))
    ((9 2) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x x-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x x-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1650)
  (parent 1093)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1649)
  (parent 1093)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1648)
  (parent 1093)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 3) (8 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-2)))))
  (label 1647)
  (parent 1092)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 1) (10 0))
    ((9 2) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y)))))))
  (label 1646)
  (parent 1092)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y)))))))
  (label 1645)
  (parent 1092)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y)))))))
  (label 1644)
  (parent 1092)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1643)
  (parent 1091)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x w)))
      (send (cat (exp (gen) l-0) (mul x w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1642)
  (parent 1091)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 3) (8 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y)))))))
  (label 1641)
  (parent 1091)
  (unrealized (4 0) (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul x (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul x (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1640)
  (parent 1091)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec l-0) w)) (l-1 l-0) (w-1 w))
    (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1639)
  (parent 1091)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) w x x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) w x x)))
      (send (cat (exp (gen) l) (mul (rec l) w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1638)
  (parent 1091)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx) (w expt)
    (x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 2) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w x-0)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1637)
  (parent 1091)
  (unrealized (4 0) (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1636)
  (parent 1086)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x x-0 (rec l-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x x-0 (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1635)
  (parent 1086)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul x (rec l) x-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul x (rec l) x-0)))
      (send (cat (exp (gen) l) (mul x (rec l) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1634)
  (parent 1086)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) y)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1633)
  (parent 1085)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x y (rec l-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1632)
  (parent 1085)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul x (rec l) y)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul x (rec l) y)))
      (send (cat (exp (gen) l) (mul x (rec l) y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1631)
  (parent 1085)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 rndx)
    (w expt) (y l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)) ((9 3) (11 0)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0) (11 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 1630)
  (parent 1084)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-2)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)) ((11 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-2)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-2)))
      (send (cat (exp (gen) y) (mul x (rec y) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1629)
  (parent 1084)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)) ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 11 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1628)
  (parent 1084)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 3) (7 0)) ((9 1) (8 0)) ((10 1) (0 0))
    ((10 1) (5 0)) ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (7 0))
  (gen-st (pv b-0 l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (8 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1627)
  (parent 1084)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 1626)
  (parent 1084)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 1625)
  (parent 1084)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)) ((11 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-2))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-0)))))
  (label 1624)
  (parent 1084)
  (unrealized (4 0) (8 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) x))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1623)
  (parent 1084)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (ga gx ga-0 gx-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-1) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-2) (l l-2) (y y-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 3) (8 0))
    ((10 1) (9 0)) ((11 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-2))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-1
            (hash (exp ga-0 l-2) (exp gx-0 y-0)))))))
  (label 1622)
  (parent 1084)
  (unrealized (4 0) (8 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx)
    (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 3) (8 0)) ((9 3) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 1621)
  (parent 1083)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)) ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1620)
  (parent 1083)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1619)
  (parent 1083)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 1618)
  (parent 1083)
  (unrealized (4 0) (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 3) (7 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (8 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) l-0))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1617)
  (parent 1083)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y)))))))
  (label 1616)
  (parent 1083)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0)))))
  (label 1615)
  (parent 1083)
  (unrealized (4 0) (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) x))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1614)
  (parent 1083)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l rndx) (w expt) (y l-0 rndx)
    (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) w y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 3) (8 0)) ((9 3) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 1613)
  (parent 1082)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)) ((10 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-1)))
      (send (cat (exp (gen) y) (mul x (rec y) l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1612)
  (parent 1082)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) l-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y)))))))
  (label 1611)
  (parent 1082)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l-0) (y-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 1610)
  (parent 1082)
  (unrealized (4 0) (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((8 3) (7 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 1) (8 0)) ((9 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-1)
    (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) l-0)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) l-0)))
      (send (cat (exp (gen) y) (mul x (rec y) l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1609)
  (parent 1082)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1608)
  (parent 1082)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l-0) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0)))))
  (label 1607)
  (parent 1082)
  (unrealized (4 0) (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) y) x))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1606)
  (parent 1082)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1605)
  (parent 1081)
  (unrealized (3 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x w)))
      (send (cat (exp (gen) l-0) (mul x w))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1604)
  (parent 1081)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-0) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-1) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w y)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y)))))))
  (label 1603)
  (parent 1081)
  (unrealized (3 0) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul x (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul x (rec l) w l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 1602)
  (parent 1081)
  (unrealized (3 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec l-0) w)) (l-1 l-0) (w-1 w))
    (gen) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 1601)
  (parent 1081)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor)
    (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 1660)
  (parent 1100)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 1661)
  (parent 1100)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 1662)
  (parent 1100)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3))
    ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1663)
  (parent 1100)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor)
    (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l) (x l-0) (x-0 l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 1664)
  (parent 1101)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 1665)
  (parent 1101)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l y x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-0))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-0)))))
  (label 1666)
  (parent 1101)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3))
    ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1667)
  (parent 1101)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor)
    (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l-1) (x-0 l))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv a-0 l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 1668)
  (parent 1102)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 y x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) x-0)) (priv-stor priv-stor-0) (l l-1) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 1669)
  (parent 1102)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor-0) (l l-1)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 1670)
  (parent 1102)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y l-0) (x l) (x-0 l-1))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y x-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-1)))))
  (label 1671)
  (parent 1102)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) x-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul x-0 l-2))) (priv-stor priv-stor-0) (l l-1)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (7 0)) ((9 1) (0 3))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y l-1) (x l) (x-0 l-0))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y x-0 l-2)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) x-0 l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 1672)
  (parent 1102)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y l rndx) (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1673)
  (parent 1103)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 1674)
  (parent 1104)
  (unrealized (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 1675)
  (parent 1105)
  (unrealized (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) l-1)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener x)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 1) (0 3)) ((8 1) (6 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))) ((recv x) (send x)))
  (label 1676)
  (parent 1106)
  (unrealized (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l l-0))) (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) l l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 0)) ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3))
    ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l l-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l l-0))))
    ((recv x) (send x)))
  (label 1677)
  (parent 1108)
  (unrealized (5 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) l-0)) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 0)) ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3))
    ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) l-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))) ((recv x) (send x)))
  (label 1678)
  (parent 1109)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 0)) ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3))
    ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 1679)
  (parent 1110)
  (unrealized (5 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul l-0 l-1))) (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener x)
  (precedes ((0 2) (5 0)) ((0 2) (9 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 3) (7 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (2 2))
    ((6 1) (0 0)) ((6 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 3))
    ((8 1) (5 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)) ((6 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul l-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-1)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1))))
    ((recv x) (send x)))
  (label 1680)
  (parent 1111)
  (unrealized (5 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor)
    (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 1681)
  (parent 1112)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 1682)
  (parent 1112)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 1683)
  (parent 1112)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3))
    ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1684)
  (parent 1112)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor)
    (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-0))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l) (x l-0) (y-0 l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 1685)
  (parent 1113)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 1686)
  (parent 1113)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-0))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) l-0)) (send (cat (exp (gen) y-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-0)))))
  (label 1687)
  (parent 1113)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (0 3))
    ((9 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-0))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y-0))))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1688)
  (parent 1113)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor)
    (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l-1))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l-1) (y-0 l))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv b-0 l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 1689)
  (parent 1114)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 y x y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) y-0)) (priv-stor priv-stor-0) (l l-1) (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 1690)
  (parent 1114)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor-0) (l l-1)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 1691)
  (parent 1114)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) y-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y l-0) (x l) (y-0 l-1))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y y-0 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) l-1)) (send (cat (exp (gen) y-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-1)))))
  (label 1692)
  (parent 1114)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) y-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul y-0 l-2))) (priv-stor priv-stor-0) (l l-1)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (3 2)) ((7 3) (6 0)) ((8 1) (7 0)) ((9 1) (0 3))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y l-1) (x l) (y-0 l-0))
  (precur (4 0) (6 0) (9 0))
  (gen-st (pv a l) (pv b l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((8 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) y-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul y-0 l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul y y-0 l-2)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) l-2)) (send (cat (exp (gen) y-0) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) y-0 l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 1693)
  (parent 1114)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-0 l-0))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
  (deflistener x)
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (7 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 1) (0 3)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-0 l-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul l-0 l-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-0 l-0)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 l-0))))
    ((recv x) (send x)))
  (label 1694)
  (parent 1117)
  (unrealized (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (ga (exp (gen) l))
    (gx (exp (gen) (mul l-1 l-1))) (priv-stor priv-stor-0) (l l-0)
    (y y))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
  (deflistener x)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 2)) ((7 2) (6 0)) ((8 1) (0 3))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (4 0) (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) l-1 l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul l-1 l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y l-1 l-1)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 l-1))))
    ((recv x) (send x)))
  (label 1695)
  (parent 1122)
  (unrealized (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 1696)
  (parent 1180)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (11 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l) (pv self l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((9 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2)))
    ((load priv-stor-2 (cat pt-8 (pv self l-2)))
      (stor priv-stor-2 (cat pt-9 "nil")) (send l-2))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1697)
  (parent 1185)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (11 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (gen-st (pv a l-0) (pv b l) (pv self l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((9 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2)))
    ((load priv-stor-2 (cat pt-8 (pv self l-2)))
      (stor priv-stor-2 (cat pt-9 "nil")) (send l-2))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1698)
  (parent 1185)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1699)
  (parent 1186)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 1700)
  (parent 1186)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1701)
  (parent 1187)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 1702)
  (parent 1187)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y)))))))
  (label 1703)
  (parent 1188)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 1704)
  (parent 1188)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l l-0)) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-2) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) l-2))
      (send (cat (exp (gen) (mul l l-0)) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-1)))))
  (label 1705)
  (parent 1189)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (y-0 l-2 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 1706)
  (parent 1189)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1707)
  (parent 1190)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 1708)
  (parent 1190)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1709)
  (parent 1191)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 1710)
  (parent 1191)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1711)
  (parent 1192)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 1712)
  (parent 1192)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l l-0)) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-2) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) l-2))
      (send (cat (exp (gen) (mul l l-0)) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-1)))))
  (label 1713)
  (parent 1193)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (x-0 l-2 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 1714)
  (parent 1193)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1715)
  (parent 1195)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1716)
  (parent 1196)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1717)
  (parent 1198)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-0) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (2 2)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l-1) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1718)
  (parent 1200)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x w (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 2)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x w (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1719)
  (parent 1202)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 rndx) (w expt) (l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-1) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l l-0)) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) l-2))
      (send (cat (exp (gen) (mul l l-0)) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1720)
  (parent 1204)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1721)
  (parent 1205)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1722)
  (parent 1205)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1723)
  (parent 1205)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1724)
  (parent 1206)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1725)
  (parent 1206)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1726)
  (parent 1206)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1727)
  (parent 1207)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1728)
  (parent 1207)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1729)
  (parent 1207)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1730)
  (parent 1208)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1731)
  (parent 1208)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1732)
  (parent 1208)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1733)
  (parent 1209)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1734)
  (parent 1209)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1735)
  (parent 1209)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1736)
  (parent 1210)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1737)
  (parent 1210)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1738)
  (parent 1210)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1739)
  (parent 1211)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1740)
  (parent 1211)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1741)
  (parent 1211)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1742)
  (parent 1212)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1743)
  (parent 1212)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1744)
  (parent 1212)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1745)
  (parent 1213)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (2 2)) ((10 1) (0 3)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1746)
  (parent 1217)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (2 2)) ((10 1) (0 3)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1747)
  (parent 1217)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 1) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1748)
  (parent 1220)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 1) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1749)
  (parent 1220)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (12 0)) ((10 2) (3 2)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((10 1) (12 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-2 (11 0)
    (ch-msg priv-stor-2 (cat pt-8 (pv self l-2))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2)))
    ((load priv-stor-2 (cat pt-8 (pv self l-2)))
      (stor priv-stor-2 (cat pt-9 "nil")) (send l-2)))
  (label 1750)
  (parent 1223)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1751)
  (parent 1226)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1752)
  (parent 1226)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1753)
  (parent 1226)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1754)
  (parent 1226)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1755)
  (parent 1227)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1756)
  (parent 1227)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1757)
  (parent 1227)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1758)
  (parent 1227)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) x))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 1759)
  (parent 1228)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) w))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 1760)
  (parent 1228)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1761)
  (parent 1229)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0))))))
  (label 1762)
  (parent 1230)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1763)
  (parent 1230)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-0) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (2 2))
    ((10 1) (0 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv b l-1) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((10 1) (0 0)) ((10 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1764)
  (parent 1231)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (2 2))
    ((10 1) (0 0)) ((10 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((10 1) (0 0)) ((10 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1765)
  (parent 1231)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 2))
    ((10 1) (3 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((10 1) (3 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1766)
  (parent 1232)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 2))
    ((10 1) (3 0)) ((10 1) (5 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (9 0) (11 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((10 1) (3 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1767)
  (parent 1232)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l l-0)) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) l-2))
      (send (cat (exp (gen) (mul l l-0)) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x (rec l-1)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1768)
  (parent 1233)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-2)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (9 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))))
  (label 1769)
  (parent 1233)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l)))
  (label 1770)
  (parent 1258)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (11 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((9 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2)))
    ((load priv-stor-2 (cat pt-8 (pv self l-2)))
      (stor priv-stor-2 (cat pt-9 "nil")) (send l-2))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1771)
  (parent 1263)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (11 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((9 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2)))
    ((load priv-stor-2 (cat pt-8 (pv self l-2)))
      (stor priv-stor-2 (cat pt-9 "nil")) (send l-2))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1772)
  (parent 1263)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1773)
  (parent 1264)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 1774)
  (parent 1264)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1775)
  (parent 1265)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 1776)
  (parent 1265)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y)))))))
  (label 1777)
  (parent 1266)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 1778)
  (parent 1266)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y-0)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-2) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-1) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-2))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-1)))))
  (label 1779)
  (parent 1267)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (y-0 l-2 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 3) (9 0)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 1780)
  (parent 1267)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1781)
  (parent 1268)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 1782)
  (parent 1268)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1783)
  (parent 1269)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 1784)
  (parent 1269)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1785)
  (parent 1270)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 1786)
  (parent 1270)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-2) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-2))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-1)))))
  (label 1787)
  (parent 1271)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (x-0 l-2 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 1788)
  (parent 1271)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1789)
  (parent 1273)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1790)
  (parent 1274)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1791)
  (parent 1276)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-0) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (2 2)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l-1) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1792)
  (parent 1278)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul x w (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (3 2)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul x w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x w (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1793)
  (parent 1280)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 rndx) (w expt) (l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) (mul x w)))
    (priv-stor priv-stor-1) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1) w)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (3 2)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul x (rec l-1) w))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1794)
  (parent 1282)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1795)
  (parent 1283)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1796)
  (parent 1283)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1797)
  (parent 1283)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1798)
  (parent 1284)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1799)
  (parent 1284)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1800)
  (parent 1284)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1801)
  (parent 1285)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1802)
  (parent 1285)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0))))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1803)
  (parent 1285)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) y-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) y-0))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y)) y-0))
      (send (cat (exp (gen) (mul (rec x) y)) y-0))))
  (label 1804)
  (parent 1286)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0)) y))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y-0)) y))
      (send (cat (exp (gen) (mul (rec x) y-0)) y))))
  (label 1805)
  (parent 1286)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) y-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y y-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y y-0)) x))
    (exp (gen) (mul (rec x) y y-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) y-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2))))
    ((recv (cat (exp (gen) (mul y y-0)) x))
      (send (cat (exp (gen) (mul y y-0)) x))))
  (label 1806)
  (parent 1286)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1807)
  (parent 1287)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1808)
  (parent 1287)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1809)
  (parent 1287)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1810)
  (parent 1288)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1811)
  (parent 1288)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1812)
  (parent 1288)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1813)
  (parent 1289)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1814)
  (parent 1289)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1815)
  (parent 1289)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) x-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y)) x-0))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) y)) x-0))
      (send (cat (exp (gen) (mul (rec x) y)) x-0))))
  (label 1816)
  (parent 1290)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) x-0)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0)) y))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul (rec x) x-0)) y))
      (send (cat (exp (gen) (mul (rec x) x-0)) y))))
  (label 1817)
  (parent 1290)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) x-0))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul y x-0)) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (3 2)) ((11 1) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul y x-0)) x))
    (exp (gen) (mul (rec x) y x-0)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x-0)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2))))
    ((recv (cat (exp (gen) (mul y x-0)) x))
      (send (cat (exp (gen) (mul y x-0)) x))))
  (label 1818)
  (parent 1290)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1819)
  (parent 1291)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (2 2)) ((10 1) (0 3)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1820)
  (parent 1295)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (2 2)) ((10 1) (0 3)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1821)
  (parent 1295)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 1) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1822)
  (parent 1298)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) l-1))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (3 2)) ((10 1) (0 3)) ((11 1) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul (rec x) y)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) l-1)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-1))
      (send (cat (exp (gen) (mul (rec x) y)) l-1)))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1823)
  (parent 1298)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) l-2))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y)) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (12 0)) ((10 2) (3 2)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((10 1) (12 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-2 (11 0)
    (ch-msg priv-stor-2 (cat pt-8 (pv self l-2))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) l-2)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y)) l-2))
      (send (cat (exp (gen) (mul (rec x) y)) l-2)))
    ((load priv-stor-2 (cat pt-8 (pv self l-2)))
      (stor priv-stor-2 (cat pt-9 "nil")) (send l-2)))
  (label 1824)
  (parent 1301)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1825)
  (parent 1304)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1826)
  (parent 1304)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y-0)))))))
  (label 1827)
  (parent 1304)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w y-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w y-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w y-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y-0))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1828)
  (parent 1304)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1829)
  (parent 1305)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1830)
  (parent 1305)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1831)
  (parent 1305)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w x-0)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w x-0))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w x-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1832)
  (parent 1305)
  (unrealized (0 3))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) x))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 1833)
  (parent 1306)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 l-1 rndx)
    (w w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) w))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) w)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 1834)
  (parent 1306)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y l l-0 rndx) (w expt)
    (l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y w))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y w))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 1835)
  (parent 1307)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0))))))
  (label 1836)
  (parent 1308)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1837)
  (parent 1308)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-0) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (2 2))
    ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv b l-1) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1838)
  (parent 1309)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (ga (exp (gen) (mul (rec l) l-0 l-1))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (2 2))
    ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (8 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y w l-1)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1839)
  (parent 1309)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 2))
    ((10 1) (3 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((10 1) (3 0)) ((10 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1840)
  (parent 1310)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y l l-0 rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y w l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (ga (exp (gen) (mul l l-0 (rec l-1)))) (gx (exp (gen) (mul w l-1)))
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 2))
    ((10 1) (3 0)) ((10 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (8 0) (9 0) (11 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((10 1) (3 0)) ((10 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-1)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-1))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul w l-1))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y w l-1)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-1 (rec w-0))) w-0))))
  (label 1841)
  (parent 1310)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l l-0 (rec l-2)))) (gx (exp (gen) x))
    (priv-stor priv-stor-1) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-2))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul x (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-2))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-2))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-2))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-2))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((load priv-stor-1 (cat pt-3 (pv b l-2)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul x (rec l-1))))
      (send (cat (exp (gen) l-1) (mul x (rec l-1)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1842)
  (parent 1311)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (x y l l-0 l-1 rndx) (w expt) (l-2 rndx) (w-0 expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y w l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b)
    (ga (exp (gen) (mul l (rec l-0) l-1))) (gx (exp (gen) (mul w l-2)))
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (3 2)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (9 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w l-2)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul w l-2))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y w l-2)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w l-2 (rec w-0))) w-0))))
  (label 1843)
  (parent 1311)
  (unrealized (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l (rec x)))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec x)) l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0))
    ((6 2) (3 0)) ((7 1) (0 3)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (9 0)) ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l))
      (send (cat (exp (gen) (rec x)) l)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1844)
  (parent 1312)
  (unrealized (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0))
    ((6 2) (3 0)) ((7 1) (0 3)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 6 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1845)
  (parent 1312)
  (unrealized (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (8 0))
  (gen-st (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 1846)
  (parent 1312)
  (unrealized (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 1847)
  (parent 1313)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 1848)
  (parent 1314)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l (rec x)))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec x)) l))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0))
    ((6 2) (3 0)) ((7 1) (0 3)) ((8 1) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (9 0)) ((6 1) (0 0)) ((6 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l))
      (send (cat (exp (gen) (rec x)) l)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y)))))))
  (label 1849)
  (parent 1316)
  (unrealized (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0))
    ((6 2) (3 0)) ((7 1) (0 3)) ((8 1) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 6 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y)))))))
  (label 1850)
  (parent 1316)
  (unrealized (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((6 1) (0 0)) ((6 1) (5 0)) ((6 2) (3 0))
    ((7 1) (0 3)) ((8 1) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (8 0))
  (gen-st (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((6 1) (0 0)) ((6 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1851)
  (parent 1316)
  (unrealized (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0))
    ((7 2) (4 0)) ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y)))))))
  (label 1852)
  (parent 1319)
  (unrealized (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0))
    ((7 2) (4 0)) ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-1))
  (precur (10 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y)))))))
  (label 1853)
  (parent 1319)
  (unrealized (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 1) (11 0))
    ((7 2) (4 0)) ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)) ((7 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 7 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y)))))))
  (label 1854)
  (parent 1319)
  (unrealized (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)) ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1855)
  (parent 1319)
  (unrealized (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0))
    ((7 2) (4 0)) ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1856)
  (parent 1320)
  (unrealized (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0))
    ((7 2) (4 0)) ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-1))
  (precur (10 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1857)
  (parent 1320)
  (unrealized (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 1) (11 0))
    ((7 2) (4 0)) ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)) ((7 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 7 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1858)
  (parent 1320)
  (unrealized (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-2))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)) ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1859)
  (parent 1320)
  (unrealized (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 1860)
  (parent 1321)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (9 0)) ((7 2) (4 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (10 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 1861)
  (parent 1322)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 rndx) (w expt) (x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb-0)
    (priv-stor priv-stor) (l l-0) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3))
    ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-1)))))
  (label 1862)
  (parent 1333)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 rndx) (w expt) (x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb-0)
    (priv-stor priv-stor-0) (l l-0) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3))
    ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l) (x-1 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-1)))))
  (label 1863)
  (parent 1333)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1864)
  (parent 1333)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-0))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) l-0)) (send (cat (exp (gen) x) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-0)))))
  (label 1865)
  (parent 1334)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0)))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-1 x-0) (w (rec x-0))) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))))
  (label 1866)
  (parent 1334)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul x (rec x-0) l-0))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) l-0)) (send (cat (exp (gen) x) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-0)))))
  (label 1867)
  (parent 1334)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1868)
  (parent 1334)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (deflistener x)
  (precedes ((0 2) (9 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((recv x) (send x)))
  (label 1869)
  (parent 1334)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (gen) (mul (rec x) x-0 l-0)))
      (send (cat (gen) (mul (rec x) x-0 l-0)))))
  (label 1870)
  (parent 1335)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 1871)
  (parent 1335)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (gen) (mul (rec x) x-0 l-0)))
      (send (cat (gen) (mul (rec x) x-0 l-0)))))
  (label 1872)
  (parent 1335)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (gen) (mul (rec x) x-0 l-1)))
      (send (cat (gen) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1873)
  (parent 1335)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-0))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) l-0)) (send (cat (exp (gen) x) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))))
  (label 1874)
  (parent 1336)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0)))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w (mul x (rec x-0) (rec x-0))) (x-2 x-0)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 1875)
  (parent 1336)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul x (rec x-0) l-0))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) l-0)) (send (cat (exp (gen) x) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))))
  (label 1876)
  (parent 1336)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1877)
  (parent 1336)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x x-0 l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1878)
  (parent 1337)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-0)))
      (l-1 l-0)) (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1879)
  (parent 1337)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1880)
  (parent 1337)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b b-0 self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 1881)
  (parent 1337)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3))
    ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1882)
  (parent 1338)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3))
    ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1883)
  (parent 1338)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 3) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-1 l-1)))))
  (label 1884)
  (parent 1338)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x x-0 l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))))
  (label 1885)
  (parent 1339)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-0)))
      (l-1 l-0)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))))
  (label 1886)
  (parent 1339)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0)))))
  (label 1887)
  (parent 1339)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1888)
  (parent 1339)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1889)
  (parent 1340)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-1)))
      (l-2 l-1)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1890)
  (parent 1340)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1891)
  (parent 1340)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1892)
  (parent 1340)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b b-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x x-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1893)
  (parent 1340)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 rndx) (w expt) (x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb-0)
    (priv-stor priv-stor) (l l-0) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3))
    ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l) (x-1 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-1)))))
  (label 1894)
  (parent 1347)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 rndx) (w expt) (x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb-0)
    (priv-stor priv-stor-0) (l l-0) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3))
    ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-1)))))
  (label 1895)
  (parent 1347)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1896)
  (parent 1347)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-0))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) l-0)) (send (cat (exp (gen) x) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-0)))))
  (label 1897)
  (parent 1348)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul x (rec x-0)))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-1 x-0) (w (rec x-0))) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))))
  (label 1898)
  (parent 1348)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul x (rec x-0) l-0))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) l-0)) (send (cat (exp (gen) x) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-0)))))
  (label 1899)
  (parent 1348)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1900)
  (parent 1348)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (deflistener x)
  (precedes ((0 2) (9 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((recv x) (send x)))
  (label 1901)
  (parent 1348)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (gen) (mul (rec x) x-0 l-0)))
      (send (cat (gen) (mul (rec x) x-0 l-0)))))
  (label 1902)
  (parent 1349)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 1903)
  (parent 1349)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0 l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (gen) (mul (rec x) x-0 l-0)))
      (send (cat (gen) (mul (rec x) x-0 l-0)))))
  (label 1904)
  (parent 1349)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0 l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (gen) (mul (rec x) x-0 l-1)))
      (send (cat (gen) (mul (rec x) x-0 l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1905)
  (parent 1349)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-0))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) l-0)) (send (cat (exp (gen) x) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))))
  (label 1906)
  (parent 1350)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0)))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w (mul x (rec x-0) (rec x-0))) (x-2 x-0)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 1907)
  (parent 1350)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul x (rec x-0) l-0))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) l-0)) (send (cat (exp (gen) x) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-0)))))
  (label 1908)
  (parent 1350)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1909)
  (parent 1350)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x x-0 l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1910)
  (parent 1351)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-0)))
      (l-1 l-0)) (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1911)
  (parent 1351)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1912)
  (parent 1351)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b b-0 self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 1913)
  (parent 1351)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3))
    ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1914)
  (parent 1352)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3))
    ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1915)
  (parent 1352)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 3) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-1 l-1)))))
  (label 1916)
  (parent 1352)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x x-0 l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) x-0 (rec l) l-0)))))
  (label 1917)
  (parent 1353)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-0)))
      (l-1 l-0)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0))))))
  (label 1918)
  (parent 1353)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-0))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-0)) (send (cat (exp (gen) x-0) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0)))))
  (label 1919)
  (parent 1353)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1920)
  (parent 1353)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1921)
  (parent 1354)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-1)))
      (l-2 l-1)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1922)
  (parent 1354)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1923)
  (parent 1354)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1924)
  (parent 1354)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b b-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x x-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (9 0)) ((9 1) (0 3)) ((10 2) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1925)
  (parent 1354)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb-0)
    (priv-stor priv-stor-0) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3))
    ((11 2) (7 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-1)))))
  (label 1926)
  (parent 1363)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb-0)
    (priv-stor priv-stor) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3))
    ((11 2) (7 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l-1) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-1)))))
  (label 1927)
  (parent 1363)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((9 1) (11 0)) ((10 1) (0 3))
    ((11 2) (7 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((9 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 1928)
  (parent 1363)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (x-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb-0)
    (priv-stor priv-stor-2) (l l-2) (x x-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (7 0))
    ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-1))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-1 l-2)))))
  (label 1929)
  (parent 1363)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-1)))))
  (label 1930)
  (parent 1364)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l-1)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-1)))))
  (label 1931)
  (parent 1364)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul x (rec x-0)))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-1 x-0) (w (rec x-0))) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))))
  (label 1932)
  (parent 1364)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-1)))))
  (label 1933)
  (parent 1364)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul x (rec x-0) l-2))) (priv-stor priv-stor) (l l)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x) (mul (rec x-0) l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) l-2)) (send (cat (exp (gen) x) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x) (mul (rec x-0) l-2)))
      (send (cat (exp (gen) x) (mul (rec x-0) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 1934)
  (parent 1364)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (deflistener x)
  (precedes ((0 2) (10 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((recv x) (send x)))
  (label 1935)
  (parent 1364)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (gen) (mul (rec x) x-0 l-1)))
      (send (cat (gen) (mul (rec x) x-0 l-1)))))
  (label 1936)
  (parent 1365)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 1937)
  (parent 1365)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (gen) (mul (rec x) x-0 l-1)))
      (send (cat (gen) (mul (rec x) x-0 l-1)))))
  (label 1938)
  (parent 1365)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) x-0 l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (gen) (mul (rec x) x-0 l-1)))
      (send (cat (gen) (mul (rec x) x-0 l-1)))))
  (label 1939)
  (parent 1365)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) x-0 l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (gen) (mul (rec x) x-0 l-2)))
      (send (cat (gen) (mul (rec x) x-0 l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 1940)
  (parent 1365)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l-1)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))))
  (label 1941)
  (parent 1366)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0)))) (priv-stor priv-stor) (l l-1)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) x)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) x)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w (mul x (rec x-0) (rec x-0))) (x-2 x-0)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) x))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) x)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) x))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) x)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) x))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 1942)
  (parent 1366)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv a-0 l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))))
  (label 1943)
  (parent 1366)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-1))) (priv-stor priv-stor) (l l-0)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) l-1)) (send (cat (exp (gen) x) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-1)))))
  (label 1944)
  (parent 1366)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul x (rec x-0) l-2))) (priv-stor priv-stor) (l l-1)
    (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-2))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x) l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul x (rec x-0) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x l-2)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) l-2)) (send (cat (exp (gen) x) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 1945)
  (parent 1366)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor-1)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1946)
  (parent 1367)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-1)))
      (l-2 l-1)) (one) (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1947)
  (parent 1367)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor-1)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (6 0) (9 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1948)
  (parent 1367)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1949)
  (parent 1367)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b a b-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x x-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor-1)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 2) (6 0)) ((7 2) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0))
    ((11 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1950)
  (parent 1367)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3))
    ((11 3) (7 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b (hash (exp ga l-1) (exp gx y)))))))
  (label 1951)
  (parent 1368)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3))
    ((11 3) (7 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l-1) (x-0 l-0) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 a (hash (exp ga l-1) (exp gx y)))))))
  (label 1952)
  (parent 1368)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b a-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((9 1) (11 0)) ((10 1) (0 3))
    ((11 3) (7 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((9 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 a-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 1953)
  (parent 1368)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 na-1 nb-0 data) (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 3) (7 0))
    ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b-1 (hash (exp ga l-2) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-1 l-2)))))
  (label 1954)
  (parent 1368)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv a-0 l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))))
  (label 1955)
  (parent 1369)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-1)))
      (l-2 l-1)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))))
  (label 1956)
  (parent 1369)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0)))))
  (label 1957)
  (parent 1369)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))))
  (label 1958)
  (parent 1369)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x x-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 1959)
  (parent 1369)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1960)
  (parent 1370)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-1)))
      (l-2 l-1)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1961)
  (parent 1370)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) x-0 (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1962)
  (parent 1370)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-1))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-1)) (send (cat (exp (gen) x-0) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1963)
  (parent 1370)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b b-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x x-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 1964)
  (parent 1370)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x x-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-2))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-2))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-2))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 2 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1965)
  (parent 1371)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x x-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor) (l l-2)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-2))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l-2) (x-0 l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-2) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 1 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-2))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor (cat pt (pv a l-2)))
      (stor priv-stor (cat pt-3 "nil")) (send l-2))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1966)
  (parent 1371)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) x-0 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec l-2)))
      (l-3 l-2)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) x-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) x-0 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) x-0 (rec l-2)))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1967)
  (parent 1371)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x x-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 9 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-2))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) x-0 (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1968)
  (parent 1371)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-2))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) x-0)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 11 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-2)) (send (cat (exp (gen) x-0) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) x-0)))
      (send (cat (exp (gen) l-2) (mul (rec x) x-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1969)
  (parent 1371)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 data) (a b a-0 b-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (l l-0 l-1 x x-0 l-2 l-3 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) x-0 l-3))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-3))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-3))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) l-3))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) x-0 (rec l-2) l-3)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-3) (self self-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0))
    ((12 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2 l-3)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) x-0 l-3))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-3)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-3))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-3)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-3))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 l-3)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) l-3)) (send (cat (exp (gen) x-0) l-3)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) x-0 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec x) x-0 (rec l-2) l-3))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv self-0 l-3)))))
  (label 1970)
  (parent 1371)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1971)
  (parent 1372)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1972)
  (parent 1373)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w (mul (rec x) (rec x-0))) (x-2 x-0)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1973)
  (parent 1373)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1974)
  (parent 1373)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1975)
  (parent 1373)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1976)
  (parent 1374)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w (mul (rec x) (rec x-0))) (x-2 x-0)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1977)
  (parent 1374)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1978)
  (parent 1374)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1979)
  (parent 1374)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0)))))
  (label 1980)
  (parent 1375)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w (mul (rec x) (rec x-0))) (x-2 x-0)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1981)
  (parent 1375)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0)))))
  (label 1982)
  (parent 1375)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1983)
  (parent 1375)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 1984)
  (parent 1375)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 1985)
  (parent 1377)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 1986)
  (parent 1382)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener x)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv x) (send x)))
  (label 1987)
  (parent 1388)
  (unrealized (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1988)
  (parent 1390)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec y))) (y-0 y)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 1989)
  (parent 1390)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y)))))))
  (label 1990)
  (parent 1390)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1991)
  (parent 1390)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y)))))))
  (label 1992)
  (parent 1391)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec y))) (y-0 y)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1993)
  (parent 1391)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 1994)
  (parent 1391)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 1995)
  (parent 1391)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 1996)
  (parent 1392)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec y))) (y-0 y)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1997)
  (parent 1392)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 1998)
  (parent 1392)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1999)
  (parent 1392)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-2)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 2000)
  (parent 1392)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (deflistener x)
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x)))) ((recv x) (send x)))
  (label 2001)
  (parent 1395)
  (unrealized (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (rec x)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener x)
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (rec x)))
      (send (cat (exp (gen) l-1) (rec x))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv x) (send x)))
  (label 2002)
  (parent 1400)
  (unrealized (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) l (rec x-0) l-0))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2003)
  (parent 1402)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (l-1 l-0) (w (mul (rec x) l-0 (rec x-0)))
      (x-2 x-0)) (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2004)
  (parent 1402)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 7 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2005)
  (parent 1402)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 2006)
  (parent 1402)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x x-0 l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2007)
  (parent 1403)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w (mul (rec x) (rec x-0) l-0)) (x-2 x-0)
      (l-1 l-0)) (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2008)
  (parent 1403)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 7 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2009)
  (parent 1403)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 2010)
  (parent 1403)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0) (pv a l))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))))
  (label 2011)
  (parent 1404)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor-0) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-1) (pv a l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (l-2 l-1) (w (mul (rec x) l-1 (rec x-0)))
      (x-2 x-0)) (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))))
  (label 2012)
  (parent 1404)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-1) (pv a l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))))
  (label 2013)
  (parent 1404)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 2014)
  (parent 1404)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (6 0)) ((9 2) (8 0)) ((10 1) (9 0))
    ((11 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-1) (pv a l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2015)
  (parent 1404)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) l (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l (rec y) l-0))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y)))))))
  (label 2016)
  (parent 1417)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l-0) (w (mul (rec x) l-0 (rec y))) (y-0 y))
    (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y)))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y)))))))
  (label 2017)
  (parent 1417)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 7 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y)))))))
  (label 2018)
  (parent 1417)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (9 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (9 0)) ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 2019)
  (parent 1417)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y)))))))
  (label 2020)
  (parent 1418)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec y) l-0)) (y-0 y) (l-1 l-0))
    (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y)))))))
  (label 2021)
  (parent 1418)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 7 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y)))))))
  (label 2022)
  (parent 1418)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((7 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 2023)
  (parent 1418)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0) (pv b-0 l))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l)))))
  (label 2024)
  (parent 1419)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor-0) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-1) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1 (rec y))) (y-0 y))
    (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 2025)
  (parent 1419)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-1) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 2026)
  (parent 1419)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2027)
  (parent 1419)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor-0)
    (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 3) (6 0)) ((9 3) (8 0)) ((10 1) (9 0))
    ((11 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-1) (pv b-0 l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-1)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2028)
  (parent 1419)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (deflistener x)
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0))
    ((7 2) (8 0)) ((8 1) (0 3)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x)))) ((recv x) (send x)))
  (label 2029)
  (parent 1421)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 2030)
  (parent 1436)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x)))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 2031)
  (parent 1436)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l) (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 2032)
  (parent 1436)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2033)
  (parent 1436)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 2) (9 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 2034)
  (parent 1436)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx gb base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3))
    ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 2035)
  (parent 1437)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx gb base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3))
    ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 2036)
  (parent 1437)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l-0) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-1 l-1)))))
  (label 2037)
  (parent 1437)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (gen) (mul (rec x) y l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (gen) (mul (rec x) y l-0)))
      (send (cat (gen) (mul (rec x) y l-0)))))
  (label 2038)
  (parent 1438)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2039)
  (parent 1438)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (gen) (mul (rec x) y l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (gen) (mul (rec x) y l-0)))
      (send (cat (gen) (mul (rec x) y l-0)))))
  (label 2040)
  (parent 1438)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (gen) (mul (rec x) y l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (gen) (mul (rec x) y l-1)))
      (send (cat (gen) (mul (rec x) y l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2041)
  (parent 1438)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))))
  (label 2042)
  (parent 1439)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x)))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (y-0 y) (w (mul y (rec x) (rec x))) (x-0 x)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2043)
  (parent 1439)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l) (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))))
  (label 2044)
  (parent 1439)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2045)
  (parent 1439)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) y (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2046)
  (parent 1440)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-0))) (l-1 l-0))
    (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2047)
  (parent 1440)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y)))
      (send (cat (exp (gen) l-0) (mul (rec x) y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2048)
  (parent 1440)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0)) ((8 1) (0 3))
    ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)) ((9 2) (8 0))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)) ((9 1) (7 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 2049)
  (parent 1440)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 locn) (l x y rndx)
    (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b a) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3))
    ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l-0) (y-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 a (hash (exp ga-0 l-0) (exp gx-0 y-0)))))))
  (label 2050)
  (parent 1441)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 locn) (l x y rndx)
    (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3))
    ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l) (y-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b (hash (exp ga-0 l-0) (exp gx-0 y-0)))))))
  (label 2051)
  (parent 1441)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 3) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l-0) (y-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2052)
  (parent 1441)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))))
  (label 2053)
  (parent 1442)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-0))) (l-1 l-0))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2054)
  (parent 1442)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y)))
      (send (cat (exp (gen) l-0) (mul (rec x) y)))))
  (label 2055)
  (parent 1442)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2056)
  (parent 1442)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2057)
  (parent 1443)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-1))) (l-2 l-1))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2058)
  (parent 1443)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2059)
  (parent 1443)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y)))
      (send (cat (exp (gen) l-1) (mul (rec x) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2060)
  (parent 1443)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x y l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2061)
  (parent 1443)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 2062)
  (parent 1450)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x)))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 2063)
  (parent 1450)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l-0)))))
  (label 2064)
  (parent 1450)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2065)
  (parent 1450)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 2) (9 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 2066)
  (parent 1450)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx gb base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3))
    ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 2067)
  (parent 1451)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx gb base)
    (priv-stor priv-stor-0 locn) (l x y rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3))
    ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 2068)
  (parent 1451)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a a-1) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-1 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-1 a-1 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-1 l-1)))))
  (label 2069)
  (parent 1451)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (gen) (mul (rec x) y l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (gen) (mul (rec x) y l-0)))
      (send (cat (gen) (mul (rec x) y l-0)))))
  (label 2070)
  (parent 1452)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2071)
  (parent 1452)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) (mul (rec x) y l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (gen) (mul (rec x) y l-0)))
      (send (cat (gen) (mul (rec x) y l-0)))))
  (label 2072)
  (parent 1452)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (gen) (mul (rec x) y l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (gen) (mul (rec x) y l-1)))
      (send (cat (gen) (mul (rec x) y l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2073)
  (parent 1452)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))))
  (label 2074)
  (parent 1453)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x)))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (y-0 y) (w (mul y (rec x) (rec x))) (x-0 x)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2075)
  (parent 1453)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l y x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-0)))))
  (label 2076)
  (parent 1453)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2077)
  (parent 1453)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (deflistener (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) y (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2078)
  (parent 1454)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-0))) (l-1 l-0))
    (one) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2079)
  (parent 1454)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y)))
      (send (cat (exp (gen) l-0) (mul (rec x) y))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2080)
  (parent 1454)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (8 0))
    ((8 1) (0 3)) ((9 1) (0 0)) ((9 1) (5 0)) ((9 2) (8 0))
    ((10 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 2081)
  (parent 1454)
  (unrealized (0 3) (6 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 locn) (l x y rndx)
    (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b a) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor) (l l-0) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3))
    ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y l) (y-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 a (hash (exp ga-0 l-0) (exp gx-0 y-0)))))))
  (label 2082)
  (parent 1455)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 locn) (l x y rndx)
    (w expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-0) (l l-0) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (4 0)) ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3))
    ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b (hash (exp ga-0 l-0) (exp gx-0 y-0)))))))
  (label 2083)
  (parent 1455)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 a-1 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 3) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2084)
  (parent 1455)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) y (rec l) l-0)))))
  (label 2085)
  (parent 1456)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-0))) (l-1 l-0))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2086)
  (parent 1456)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-0)) (send (cat (exp (gen) y) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y)))
      (send (cat (exp (gen) l-0) (mul (rec x) y)))))
  (label 2087)
  (parent 1456)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2088)
  (parent 1456)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2089)
  (parent 1457)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-1))) (l-2 l-1))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2090)
  (parent 1457)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2091)
  (parent 1457)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y)))
      (send (cat (exp (gen) l-1) (mul (rec x) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2092)
  (parent 1457)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x y l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (9 0)) ((9 1) (0 3)) ((10 2) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2093)
  (parent 1457)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1)))))
  (label 2094)
  (parent 1466)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1)))))
  (label 2095)
  (parent 1466)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x)))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-1) (x l))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 2096)
  (parent 1466)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-1) (x l))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-1)))))
  (label 2097)
  (parent 1466)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul y (rec x) l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y l-1) (x l))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 2098)
  (parent 1466)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx)
    (w expt) (l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 2) (10 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 2099)
  (parent 1466)
  (unrealized (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3))
    ((11 2) (7 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-0) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-0)))))
  (label 2100)
  (parent 1467)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3))
    ((11 2) (7 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-1) (y l-0) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-0)))))
  (label 2101)
  (parent 1467)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a b-0) (b b-1) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((9 1) (11 0)) ((10 1) (0 3))
    ((11 2) (7 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((9 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b-0 b-1 (exp (gen) x-0)))))
  (label 2102)
  (parent 1467)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y rndx) (w expt) (x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (7 0))
    ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (y l-1) (x-0 l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-1 l-2)))))
  (label 2103)
  (parent 1467)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (gen) (mul (rec x) y l-1)))
      (send (cat (gen) (mul (rec x) y l-1)))))
  (label 2104)
  (parent 1468)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2105)
  (parent 1468)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (gen) (mul (rec x) y l-1)))
      (send (cat (gen) (mul (rec x) y l-1)))))
  (label 2106)
  (parent 1468)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (gen) (mul (rec x) y l-1)))
      (send (cat (gen) (mul (rec x) y l-1)))))
  (label 2107)
  (parent 1468)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (gen) (mul (rec x) y l-2)))
      (send (cat (gen) (mul (rec x) y l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 2108)
  (parent 1468)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))))
  (label 2109)
  (parent 1469)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x)))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-0) (x l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (y-0 y) (w (mul y (rec x) (rec x))) (x-0 x)) (one)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2110)
  (parent 1469)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l) (x l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv b-0 l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))))
  (label 2111)
  (parent 1469)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 y x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y l-1) (x l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-1)))))
  (label 2112)
  (parent 1469)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 y x l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul y (rec x) l-2))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x) l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y l-0) (x l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x) l-2)))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 2113)
  (parent 1469)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (6 0) (9 0))
  (gen-st (pv b-0 l) (pv self l-0))
  (facts (neq self b) (undisclosed l-1))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b-0 l)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2114)
  (parent 1470)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b-0 l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-1))) (l-2 l-1))
    (one) (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) y))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2115)
  (parent 1470)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (6 0) (9 0))
  (gen-st (pv b-0 l-1) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-1))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2116)
  (parent 1470)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b-0 l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y)))
      (send (cat (exp (gen) l-1) (mul (rec x) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2117)
  (parent 1470)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b a b-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x y l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (6 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (3 0)) ((7 3) (6 0)) ((7 3) (9 0)) ((8 1) (7 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0))
    ((11 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b-0 l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (7 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (6 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2118)
  (parent 1470)
  (unrealized (0 3) (6 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-0) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3))
    ((11 3) (7 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-0) (y-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 2119)
  (parent 1471)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b a) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (11 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 3) (7 0)) ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3))
    ((11 3) (7 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-1) (y l-0) (y-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 a (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 2120)
  (parent 1471)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx ga-0 gx-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y rndx) (w expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-0) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-1) (l l-1) (y y-0))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((9 1) (11 0)) ((10 1) (0 3))
    ((11 3) (7 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((9 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-0 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-0
            (hash (exp ga-0 l-1) (exp gx-0 y-0)))))))
  (label 2121)
  (parent 1471)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (ga gx ga-0 gx-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y rndx) (w expt) (y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-1) (b b-1) (ga ga-0)
    (gx gx-0) (priv-stor priv-stor-2) (l l-2) (y y-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 3) (7 0))
    ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (y l-1) (y-0 l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)) ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga-0 (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx-0))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-1 b-1
            (hash (exp ga-0 l-2) (exp gx-0 y-0))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-1 l-2)))))
  (label 2122)
  (parent 1471)
  (unrealized (4 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (7 0) (10 0))
  (gen-st (pv a l-1) (pv b-0 l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))))
  (label 2123)
  (parent 1472)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-1))) (l-2 l-1))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 2124)
  (parent 1472)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y)))
      (send (cat (exp (gen) l-1) (mul (rec x) y)))))
  (label 2125)
  (parent 1472)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))))
  (label 2126)
  (parent 1472)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x y l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 2127)
  (parent 1472)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2128)
  (parent 1473)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-1))) (l-2 l-1))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2129)
  (parent 1473)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-0) (exp gx y))))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2130)
  (parent 1473)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-1)) (send (cat (exp (gen) y) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y)))
      (send (cat (exp (gen) l-1) (mul (rec x) y))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2131)
  (parent 1473)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x y l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (9 0)) ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y))))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2132)
  (parent 1473)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x y l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-2))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-2))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-2))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 2 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2133)
  (parent 1474)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x y l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l-2)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-2))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-2) (y l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-2) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 1 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-2))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor (cat pt (pv a l-2)))
      (stor priv-stor (cat pt-3 "nil")) (send l-2))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2134)
  (parent 1474)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w (mul (rec x) y (rec l-2))) (l-3 l-2))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) y)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) y))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2135)
  (parent 1474)
  (unrealized (0 3) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x y l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 9 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-2))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1) l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2136)
  (parent 1474)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (7 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 11 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-2)) (send (cat (exp (gen) y) l-2)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y)))
      (send (cat (exp (gen) l-2) (mul (rec x) y))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2137)
  (parent 1474)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (a b a-0 b-0 self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (l l-0 l-1 x y l-2 l-3 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) y l-3))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-3))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-3))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) l-3))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2) l-3)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-3) (self self-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 3) (7 0))
    ((8 3) (10 0)) ((9 1) (8 0)) ((10 1) (0 3)) ((11 2) (10 0))
    ((12 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2 l-3)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y l-3))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-3)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-3))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y l-3)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-3))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y l-3)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) l-3)) (send (cat (exp (gen) y) l-3)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2) l-3))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv self-0 l-3)))))
  (label 2138)
  (parent 1474)
  (unrealized (0 3) (7 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) l (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2139)
  (parent 1475)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (l-1 l-0) (w (mul (rec x) l-0 (rec x-0)))
      (x-2 x-0)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2140)
  (parent 1475)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2141)
  (parent 1475)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2142)
  (parent 1475)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x x-0 l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2143)
  (parent 1476)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w (mul (rec x) (rec x-0) l-0)) (x-2 x-0)
      (l-1 l-0)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2144)
  (parent 1476)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2145)
  (parent 1476)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2146)
  (parent 1476)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1) (pv a-0 l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l)))))
  (label 2147)
  (parent 1477)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (l-2 l-1) (w (mul (rec x) l-1 (rec x-0)))
      (x-2 x-0)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0)))))
  (label 2148)
  (parent 1477)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0)))))
  (label 2149)
  (parent 1477)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2150)
  (parent 1477)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 2) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 2151)
  (parent 1477)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) l (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) l (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 2152)
  (parent 1490)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-1 l-0) (w (mul (rec x) l-0 (rec y))) (y-0 y))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y)))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y)))))))
  (label 2153)
  (parent 1490)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y)))))))
  (label 2154)
  (parent 1490)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2155)
  (parent 1490)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (x y l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y)))))))
  (label 2156)
  (parent 1491)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec y) l-0)) (y-0 y) (l-1 l-0))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 2157)
  (parent 1491)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 2158)
  (parent 1491)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 1) (0 3)) ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2159)
  (parent 1491)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1) (pv b-0 l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l)))))
  (label 2160)
  (parent 1492)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1 (rec y))) (y-0 y))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 2161)
  (parent 1492)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0)))))
  (label 2162)
  (parent 1492)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2163)
  (parent 1492)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 3) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)) ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))))
  (label 2164)
  (parent 1492)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (deflistener x)
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (0 3))
    ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x)))) ((recv x) (send x)))
  (label 2165)
  (parent 1494)
  (unrealized (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l) (exp gx y)))))))
  (label 2166)
  (parent 1503)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1 (rec y))) (y-0 y))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 2167)
  (parent 1503)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-1) (exp gx y)))))))
  (label 2168)
  (parent 1503)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 2169)
  (parent 1503)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2170)
  (parent 1503)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-1) (exp gx y)))))))
  (label 2171)
  (parent 1504)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x l-1 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-2 l-1) (w (mul (rec x) l-1 (rec y))) (y-0 y))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 2172)
  (parent 1504)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) l-0 (rec y) l-1))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l) (exp gx y)))))))
  (label 2173)
  (parent 1504)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 2174)
  (parent 1504)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2175)
  (parent 1504)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-1) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-0) (exp gx y)))))))
  (label 2176)
  (parent 1505)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec y) l-1)) (y-0 y) (l-2 l-1))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y)))))))
  (label 2177)
  (parent 1505)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-0) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-0 l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-0) (exp gx y)))))))
  (label 2178)
  (parent 1505)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y)))))))
  (label 2179)
  (parent 1505)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x y l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-1 l-2))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 self gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2180)
  (parent 1505)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-2))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-2))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l-2))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 2 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-0)))))
  (label 2181)
  (parent 1506)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l-2)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-0) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-2) (y l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-2) (pv b-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 1 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-2))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor (cat pt (pv a l-2)))
      (stor priv-stor (cat pt-3 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-0) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-0)))))
  (label 2182)
  (parent 1506)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-2 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (l-3 l-2) (w (mul (rec x) l-2 (rec y))) (y-0 y))
    (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-2 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) l-2 (rec y)))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-1)))))
  (label 2183)
  (parent 1506)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y) l-2 l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 8 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y) l-2 l-2)))
      (send (cat (exp (gen) y) (mul (rec x) (rec y) l-2 l-2))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-1)))))
  (label 2184)
  (parent 1506)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 11 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2)))
      (send (cat (exp (gen) y) (mul (rec x) l-1 (rec y) l-2))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-2)))))
  (label 2185)
  (parent 1506)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (a b self a-0 b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (l l-0 l-1 x l-2 y l-3 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2 l-3))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) y) (mul (rec x) l-2 (rec y) l-3)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-3) (self self-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (7 0)) ((10 3) (9 0)) ((11 1) (10 0))
    ((12 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1 l-2 l-3)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2 l-3))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec x) l-2 (rec y) l-3)))
      (send (cat (exp (gen) y) (mul (rec x) l-2 (rec y) l-3))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-1))))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv self-0 l-3)))))
  (label 2186)
  (parent 1506)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2187)
  (parent 1507)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (l-2 l-1) (w (mul (rec x) l-1 (rec x-0)))
      (x-2 x-0)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2188)
  (parent 1507)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2189)
  (parent 1507)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2190)
  (parent 1507)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2191)
  (parent 1507)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2192)
  (parent 1508)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (l-2 l-1) (w (mul (rec x) l-1 (rec x-0)))
      (x-2 x-0)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2193)
  (parent 1508)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-0 (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2194)
  (parent 1508)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2195)
  (parent 1508)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2196)
  (parent 1508)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2197)
  (parent 1509)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w (mul (rec x) (rec x-0) l-1)) (x-2 x-0)
      (l-2 l-1)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2198)
  (parent 1509)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x x-0 l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-0 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-0 l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2199)
  (parent 1509)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 8 ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2200)
  (parent 1509)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x x-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-2)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 1) (10 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0))
    ((11 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-1 l-2))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2201)
  (parent 1509)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-2))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-2))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-2))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 2 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-0)))))
  (label 2202)
  (parent 1510)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l-2)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-2))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l-2) (x-0 l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-2) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 1 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-2))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor (cat pt (pv a l-2)))
      (stor priv-stor (cat pt-3 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-0)))))
  (label 2203)
  (parent 1510)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-2 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (l-3 l-2) (w (mul (rec x) l-2 (rec x-0)))
      (x-2 x-0)) (one) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) l-2 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-1)))))
  (label 2204)
  (parent 1510)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-2 l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 8 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-2 l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0) l-2 l-2))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-1)))))
  (label 2205)
  (parent 1510)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1 l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-2))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 11 ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1 l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-1 (rec x-0) l-2))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-2)))))
  (label 2206)
  (parent 1510)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 data) (a b self a-0 b-0 self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (l l-0 l-1 x l-2 x-0 l-3 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2 l-3))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) l-2 (rec x-0) l-3)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-3) (self self-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0)) ((11 1) (10 0))
    ((12 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2 l-3)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2 l-3))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 l-3)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) l-2 (rec x-0) l-3)))
      (send (cat (exp (gen) x-0) (mul (rec x) l-2 (rec x-0) l-3))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-1))))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv self-0 l-3)))))
  (label 2207)
  (parent 1510)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (rec x)))
  (deflistener x)
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (4 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (rec x) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (rec x)))
      (send (cat (exp (gen) l-1) (rec x)))) ((recv x) (send x)))
  (label 2208)
  (parent 1533)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (3 0)) ((8 1) (0 3)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (6 0) (9 0))
  (gen-st (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2209)
  (parent 1551)
  (unrealized (8 0) (9 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (3 0)) ((8 1) (0 3)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 2210)
  (parent 1551)
  (unrealized (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (3 0)) ((8 1) (0 3)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (gen) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 2211)
  (parent 1551)
  (unrealized (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l (rec x)))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec x)) l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (3 0)) ((8 1) (0 3)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (9 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l))
      (send (cat (exp (gen) (rec x)) l)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))))
  (label 2212)
  (parent 1551)
  (unrealized (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (3 0)) ((8 1) (0 3)) ((9 1) (8 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-1))
  (precur (6 0) (9 0))
  (gen-st (pv self l-0) (pv b-0 l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a ga (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 2213)
  (parent 1551)
  (unrealized (8 0) (9 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (3 0)) ((7 2) (9 0)) ((8 1) (0 3)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 7 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))))
  (label 2214)
  (parent 1551)
  (unrealized (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((5 2) (8 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 1) (5 0))
    ((7 2) (3 0)) ((8 1) (0 3)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2215)
  (parent 1551)
  (unrealized (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0))
    ((8 2) (4 0)) ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-2))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-1) (pv b-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-2) (exp gx y)))))))
  (label 2216)
  (parent 1552)
  (unrealized (9 0) (11 0) (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-2) (l l-2) (x x-0))
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0))
    ((8 2) (4 0)) ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-2))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-1) (pv a-0 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2217)
  (parent 1552)
  (unrealized (9 0) (11 0) (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0))
    ((8 2) (4 0)) ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 2218)
  (parent 1552)
  (unrealized (9 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0))
    ((8 2) (4 0)) ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (11 0))
  (gen-st (pv a l-1) (pv self l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 2219)
  (parent 1552)
  (unrealized (9 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0))
    ((8 2) (4 0)) ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))))
  (label 2220)
  (parent 1552)
  (unrealized (9 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-1) (l l-1)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (10 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (9 0)) ((7 2) (3 0)) ((8 1) (0 3))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 0)) ((11 1) (5 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (6 0) (10 0))
  (gen-st (pv self l-0) (pv self-0 l-1))
  (facts (neq self-0 b) (undisclosed l))
  (leads-to ((7 1) (9 0)) ((11 1) (0 0)) ((11 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-0))
      (send (cat (exp (gen) (rec x)) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2221)
  (parent 1552)
  (unrealized (8 0) (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0))
    ((8 2) (4 0)) ((8 2) (11 0)) ((9 1) (0 3)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 8 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))))
  (label 2222)
  (parent 1552)
  (unrealized (9 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l-1 (rec x)))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0))
    ((8 2) (4 0)) ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-1 (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) l-1))
      (send (cat (exp (gen) (rec x)) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2223)
  (parent 1552)
  (unrealized (9 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb-0)
    (priv-stor priv-stor) (l l-0) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-1)))))
  (label 2224)
  (parent 1553)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb-0)
    (priv-stor priv-stor-0) (l l-0) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l) (x-1 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-1)))))
  (label 2225)
  (parent 1553)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 2) (8 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2226)
  (parent 1553)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 2227)
  (parent 1557)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 2228)
  (parent 1557)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 3) (8 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-1 l-1)))))
  (label 2229)
  (parent 1557)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3))
    ((11 2) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 2230)
  (parent 1560)
  (unrealized (4 0) (8 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 init 3) (exp (gen) x-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 2231)
  (parent 1560)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) w)) (x-2 x-0) (w-1 w))
    (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2232)
  (parent 1560)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul w x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 2233)
  (parent 1560)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 2) (7 0)) ((8 2) (9 0))
    ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0)) ((10 1) (8 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((10 1) (0 0)) ((10 1) (5 0)) ((10 1) (8 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-0))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2234)
  (parent 1560)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3))
    ((11 3) (8 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l-0) (y l-1))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y)))))))
  (label 2235)
  (parent 1560)
  (unrealized (4 0) (8 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-0))))
    ((load priv-stor (cat pt (pv a l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 2236)
  (parent 1560)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2237)
  (parent 1560)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb-0)
    (priv-stor priv-stor) (l l-0) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l-0) (x-0 l) (x-1 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-1)))))
  (label 2238)
  (parent 1561)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data) (a b b-0 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 locn) (l x x-0 x-1 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb-0)
    (priv-stor priv-stor-0) (l l-0) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-1)))))
  (label 2239)
  (parent 1561)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 2) (8 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2240)
  (parent 1561)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l-0) (x-0 l) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 2241)
  (parent 1565)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval) (gb ga gx base)
    (priv-stor priv-stor-0 locn) (l x x-0 y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((10 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 2242)
  (parent 1565)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 3) (8 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-1 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-1 l-1)))))
  (label 2243)
  (parent 1565)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3))
    ((11 2) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 2244)
  (parent 1568)
  (unrealized (4 0) (8 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 9 init 3) (exp (gen) x-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 2245)
  (parent 1568)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l l-0 x x-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) w)) (x-2 x-0) (w-1 w))
    (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2246)
  (parent 1568)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l) (x x))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul w x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 2247)
  (parent 1568)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l) (x x-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (3 0)) ((8 2) (7 0))
    ((8 2) (9 0)) ((9 1) (0 3)) ((10 1) (0 0)) ((10 1) (5 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((1 1) (8 0)) ((10 1) (0 0)) ((10 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-0))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2248)
  (parent 1568)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 na-1 nb-0 data)
    (a b b-0 a-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-0) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3))
    ((11 3) (8 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-1 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-1 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-0 b-1 (hash (exp ga l-1) (exp gx y)))))))
  (label 2249)
  (parent 1568)
  (unrealized (4 0) (8 0) (11 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x x-0 rndx) (w expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 2250)
  (parent 1568)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (9 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2251)
  (parent 1568)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (9 0)) ((11 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-0) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b (hash (exp ga l-1) (exp gx y)))))))
  (label 2252)
  (parent 1569)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (9 0)) ((11 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l-1) (x-0 l-0) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 a (hash (exp ga l-1) (exp gx y)))))))
  (label 2253)
  (parent 1569)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b a-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)) ((10 1) (11 0)) ((11 3) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)) ((10 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 a-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 a-0 (hash (exp ga l-1) (exp gx y)))))))
  (label 2254)
  (parent 1569)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 na-1 nb-0 data) (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) y)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)) ((11 3) (8 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-2))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)) ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b-1 (hash (exp ga l-2) (exp gx y))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-1 l-2)))))
  (label 2255)
  (parent 1569)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-1) (a b) (b b-1) (gb gb-0)
    (priv-stor priv-stor-0) (l l-1) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (9 0)) ((11 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 b b-1 (exp (gen) x-1)))))
  (label 2256)
  (parent 1570)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l-1) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 3 (na na-1) (a a) (b b-1) (gb gb-0)
    (priv-stor priv-stor) (l l-1) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((10 1) (9 0)) ((11 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l-1) (x-0 l-0) (x-1 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a b-1 (exp (gen) x-1)))))
  (label 2257)
  (parent 1570)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 b-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb gb-0 base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 x-1 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a a-0) (b b-1) (gb gb-0)
    (priv-stor priv-stor-1) (l l-1) (x x-1))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)) ((10 1) (11 0)) ((11 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-1))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)) ((10 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-0 b-1 (exp (gen) x-1)))))
  (label 2258)
  (parent 1570)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 x-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) x-1)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb-0)
    (priv-stor priv-stor-2) (l l-2) (x x-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0)) ((9 2) (8 0))
    ((10 1) (9 0)) ((11 2) (8 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-2))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)) ((12 1) (11 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-1 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) x-1)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-1))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-1 l-2)))))
  (label 2259)
  (parent 1570)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb gb-0 base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 rndx) (w expt) (x-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 3 (na na-1) (a a-1) (b b-1) (gb gb-0)
    (priv-stor priv-stor-2) (l l-2) (x x-1))
  (precedes ((0 2) (8 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (11 0)) ((10 1) (9 0)) ((11 1) (0 3))
    ((12 2) (8 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 na-1 l l-0 l-1)
  (uniq-gen x x-0 x-1)
  (absent (x l) (x-0 l-1) (x-1 l-2))
  (precur (7 0) (8 0) (11 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv a-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-1) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((load priv-stor-2 (cat pt-6 (pv a-1 l-2)))
      (recv (sig (body b-1 gb-0 (pubk "sig" b-1)) (privk "sig" b-1)))
      (send (cat na-1 a-1 b-1 (exp (gen) x-1)))))
  (label 2260)
  (parent 1577)
  (unrealized (4 0) (8 0) (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (11 0)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0) (11 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 9 init 3) (exp (gen) x-1) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 2261)
  (parent 1577)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x x-0 rndx)
    (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (8 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (11 0)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0) (11 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) w)) (x-2 x-0) (w-1 w))
    (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 2262)
  (parent 1577)
  (unrealized (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 2) (8 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (11 0)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (7 0) (8 0) (11 0))
  (gen-st (pv a l-1) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-1) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul w x-0 x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w x-0 x-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 2263)
  (parent 1577)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b a b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor-1) (l l-1) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (10 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 2) (7 0)) ((8 2) (10 0))
    ((9 1) (8 0)) ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (5 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-1) (x-0 l-0))
  (precur (6 0) (7 0) (10 0))
  (gen-st (pv a l-0) (pv self l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (8 0)) ((11 1) (0 0)) ((11 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2264)
  (parent 1577)
  (unrealized (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 na-1 nb-0 data)
    (a b a-0 b-0 a-1 b-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval) (gb ga gx base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 l-2 x x-0 rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-1) (nb nb-0) (a a-1) (b b-1) (ga ga) (gx gx)
    (priv-stor priv-stor-2) (l l-2) (y y))
  (precedes ((0 2) (8 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (11 0)) ((10 1) (9 0)) ((11 1) (0 3))
    ((12 3) (8 0)) ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 nb-0 l l-0 l-1)
  (uniq-gen x x-0 y)
  (absent (x l) (x-0 l-1) (y l-2))
  (precur (7 0) (8 0) (11 0))
  (gen-st (pv a l) (pv a-0 l-1) (pv b-1 l-2))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-2 (cat pt-6 (pv b-1 l-2)))
      (recv (sig (body a-1 ga (pubk "sig" a-1)) (privk "sig" a-1)))
      (recv (cat na-1 a-1 b-1 gx))
      (send
        (cat (exp (gen) y)
          (enc na-1 nb-0 a-1 b-1 (hash (exp ga l-2) (exp gx y)))))))
  (label 2265)
  (parent 1577)
  (unrealized (4 0) (8 0) (12 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x x-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-0) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (8 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (11 0)) ((10 1) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0) (11 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 2266)
  (parent 1577)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b b-0 self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn)
    (l l-0 x x-0 rndx) (w expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-1))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (10 0)) ((10 1) (0 3)) ((11 1) (9 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0) (10 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-1)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-1))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2267)
  (parent 1577)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x x-0 rndx) (w expt) (l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w l-2))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w l-2)))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 2) (8 0)) ((9 2) (11 0)) ((10 1) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1 l-2)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0) (11 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((10 1) (9 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w l-2)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w l-2)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w l-2))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2268)
  (parent 1577)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 2) (0 3)) ((4 1) (0 3)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 2269)
  (parent 1578)
  (realized)
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 1) (0 3)) ((10 2) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2270)
  (parent 1586)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 1) (0 3)) ((10 2) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2271)
  (parent 1586)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval) (gb base)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 2) (8 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-1))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2272)
  (parent 1586)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))))
  (label 2273)
  (parent 1587)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (w (mul (rec x) (rec x))) (x-0 x))
    (one) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 2274)
  (parent 1587)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-0)))))
  (label 2275)
  (parent 1587)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2276)
  (parent 1587)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (w expt) (l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) w))
  (deflistener x)
  (precedes ((0 2) (9 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul w x x) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w x x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w)))
    ((recv x) (send x)))
  (label 2277)
  (parent 1587)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul (rec x) l-0)))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul (rec x) l-0)))
      (send (cat (gen) (mul (rec x) l-0)))))
  (label 2278)
  (parent 1588)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 2279)
  (parent 1588)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul (rec x) l-0)))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul (rec x) l-0)))
      (send (cat (gen) (mul (rec x) l-0)))))
  (label 2280)
  (parent 1588)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul (rec x) l-1)))
      (send (cat (gen) (mul (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2281)
  (parent 1588)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) w)) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) w))
  (deflistener x)
  (precedes ((0 2) (9 0)) ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul x w) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) w)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))) ((recv x) (send x)))
  (label 2282)
  (parent 1588)
  (unrealized (10 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (7 0) (8 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) (rec l) l-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2283)
  (parent 1589)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec l-0))) (l-1 l-0)) (one)
    (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb self b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2284)
  (parent 1589)
  (unrealized (0 3) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 9 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2285)
  (parent 1589)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (8 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((9 1) (0 0)) ((9 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 2286)
  (parent 1589)
  (unrealized (0 3) (7 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l-0) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a) (ga ga) (gx gx)
    (priv-stor priv-stor) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((1 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 1) (0 3)) ((10 3) (8 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 a gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a (hash (exp ga l-0) (exp gx y)))))))
  (label 2287)
  (parent 1590)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (ga gx base)
    (priv-stor priv-stor-0 locn) (l x rndx) (w expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b) (ga ga) (gx gx)
    (priv-stor priv-stor-0) (l l-0) (y y))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (4 0)) ((9 1) (0 3)) ((10 3) (8 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b (hash (exp ga l-0) (exp gx y)))))))
  (label 2288)
  (parent 1590)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (ga gx base) (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx)
    (w expt) (y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w y))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0) (ga ga) (gx gx)
    (priv-stor priv-stor-1) (l l-1) (y y))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 3) (8 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y l-1))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv (sig (body a-0 ga (pubk "sig" a-0)) (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 gx))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0 (hash (exp ga l-1) (exp gx y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2289)
  (parent 1590)
  (unrealized (4 0) (8 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l-0)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-0))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec x) (rec l) l-0)))))
  (label 2290)
  (parent 1591)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec l-0))) (l-1 l-0)) (one)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0))))))
  (label 2291)
  (parent 1591)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-0))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (rec x)))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 2) l-1 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (rec x)))
      (send (cat (exp (gen) l-0) (rec x)))))
  (label 2292)
  (parent 1591)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))))
  (label 2293)
  (parent 1591)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l-1)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-1))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l-1))
  (facts (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 2) l-2 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2294)
  (parent 1592)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (rec x))) (priv-stor priv-stor) (l l) (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec l-1))) (l-2 l-1)) (one)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (rec x))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (gen))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2295)
  (parent 1592)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-1))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 2) l-2 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec x) (rec l-0) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2296)
  (parent 1592)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-1))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (rec x)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 10 ltx-gen 2) l-2 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-1))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-1)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-1))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (rec x)))
      (send (cat (exp (gen) l-1) (rec x))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2297)
  (parent 1592)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 x l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul (rec x) l-2))) (priv-stor priv-stor) (l l)
    (x x))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) (rec l-1) l-2)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (8 0)) ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (4 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1 l-2)
  (uniq-gen x)
  (absent (x l))
  (precur (7 0) (8 0) (9 0))
  (gen-st (pv a l))
  (facts (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-2))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-2)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-2))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec x) (rec l-1) l-2))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self-0 l-2)))))
  (label 2298)
  (parent 1592)
  (unrealized (0 3) (8 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (x l rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul x (rec l) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a b) (b b-0) (gb gb)
    (priv-stor priv-stor) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0))
    ((8 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (10 0)) ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul x (rec l) w x-0)))
      (send (cat (exp (gen) l) (mul x (rec l) w x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2299)
  (parent 1599)
  (unrealized (3 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (gb base) (priv-stor priv-stor-0 locn)
    (l x rndx) (w expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x w x-0 (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a self) (b b-0) (gb gb)
    (priv-stor priv-stor-0) (l l-0) (x x-0))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)) ((8 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x w x-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul x w x-0 (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2300)
  (parent 1599)
  (unrealized (3 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (gb base) (priv-stor priv-stor-0 priv-stor-1 locn) (l x l-0 rndx)
    (w expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul w x-0))) (priv-stor priv-stor-0) (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 3 (na na-0) (a a) (b b-0) (gb gb)
    (priv-stor priv-stor-1) (l l-1) (x x-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (7 0)) ((10 2) (9 0))
    ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l-0) (x-0 l-1))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0) (pv a l-1))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)) ((11 1) (10 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w x-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w x-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul x (rec l-0) w x-0)))
      (send (cat (exp (gen) l-0) (mul x (rec l-0) w x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv (sig (body b-0 gb (pubk "sig" b-0)) (privk "sig" b-0)))
      (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 2301)
  (parent 1599)
  (unrealized (3 0) (7 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l-0))
    (gy (exp (gen) (mul l (rec x) l-0))) (priv-stor priv-stor-0) (l l)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l (rec x) (rec x) l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l))
  (facts (neq self b) (undisclosed l-0))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l (rec x) l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 (pv self l)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l (rec x) (rec x) l-0)))
      (send (cat (exp (gen) x) (mul l (rec x) (rec x) l-0)))))
  (label 2302)
  (parent 1600)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x)))) (priv-stor priv-stor-0) (l l-0)
    (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x) (rec x))))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test
    (contracted (w (mul l-0 (rec x) (rec x))) (l-1 l-0) (x-0 x)) (one)
    (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x) (rec x))))))
  (label 2303)
  (parent 1600)
  (unrealized (0 3) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul (rec x) l-0 l-0))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x) l-0 l-0)))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) l-0 l-0))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x) l-0 l-0)))
      (send (cat (exp (gen) x) (mul (rec x) (rec x) l-0 l-0)))))
  (label 2304)
  (parent 1600)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (gb (exp (gen) l))
    (gy (exp (gen) (mul l-0 (rec x) l-1))) (priv-stor priv-stor-0)
    (l l-0) (x x))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (7 0)) ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((8 1) (0 0)) ((8 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l-0))
  (precur (6 0) (7 0) (9 0))
  (gen-st (pv self l-0))
  (facts (neq self b) (undisclosed l))
  (leads-to ((8 1) (0 0)) ((8 1) (5 0)))
  (rule trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul l-0 (rec x) l-1))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 l-1)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1)))
      (send (cat (exp (gen) x) (mul l-0 (rec x) (rec x) l-1))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))))
  (label 2305)
  (parent 1600)
  (unrealized (0 3) (7 0) (9 0))
  (aborted)
  (comment "aborted"))
